#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan  2 23:10:10 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2001-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                       initial after the name) 
"""

import numpy as np                       # matrix management
import matplotlib.pyplot as plt          # plots
import matplotlib.widgets as wdg         # plots widgets

def demodAnalytic (signal,fdemod='interactive',method='diff'):
    """
    Retreive the phase and the frequency by using the analytic signal phase
    around a given frequency band.

    Parameters
    ----------
    signal : vector
        signal
        
    fdemod : str or vector, optional
        Can be a vector of 2 frequencies or a string :
            
            * String version:
                * 'interactive' : Display the Fourier Transform of r_signal and ask the user for the bands.
                * 'auto'        : Let the function choose the filter (can be only use for tacho signal and not working in all case !!!)
     
            * Vector version : a 2 elements vector containing the low and high normalized frequency for extracting the speed information.
                      
        Optional, 'interactive' by default.
              
    method  : str, optional
        Method for estimation of the instantaneous frequency :
            
            * 'diff'    :
                freq(n)=[phase(n+1)-phase(n)]/(2*pi) [unwrapping phase]
            
            * 'product' :
                freq(n)={angle [-sa(n+1).sa*(n-1)]+pi}/(4*pi) where sa is analytic signal
                
                equivalent to compute [phase (n+1)-phase (n-1)]/(4*pi) 
                
                (come from instfreq of Toolbox Temps Fréquence)
                
                approximation freq(1)=freq(2)
                  
        Optionnal, 'diff' by default

    Returns
    -------
    freq : vector
        instantaneous frequency
        
    phase : vector
        instantaneous phase
        
    fdemod : vector
        value of fdemod (useful if fdemod='auto' or 'interactive')

    Example
    -------
    
    >>> import fbonnardot as fb
    >>> import numpy as np; import matplotlib.pyplot as plt
    >>> t=np.arange(0,10,0.001)
    >>> phi=2*np.pi*(50*t+2*t**2); sig=np.sin(phi)
    >>> freq,phase,__=fb.demodAnalytic(sig)
    >>> plt.figure()
    >>> plt.subplot(2,1,1); plt.plot(t,sig)
    >>> plt.subplot(2,1,2); plt.plot(t,freq)

    Note
    ----
    Be careful of border effects.

    """

    # Creation              : Monday 16 July 2001      (MATLAB version)
    # Modifications         : Wednesday 2 January 2018 (Translation to Python 3.6)
    #                         Thursday 6 February 2019 (Docstring in NumPy format)
    #                         Wednesday 1st April 2020 (Auto-test)
    # Version               : 2.1 i

    # **************************************************************************
    # * Look at the parameters                                                 *
    # **************************************************************************
    
    l=len (signal)
    
    # Remove average to suppress DC peak
    signal=signal-np.mean (signal)
    
    # **************************************************************************
    # * Try to find a band for demodulation                                    *
    # **************************************************************************
    if np.size(fdemod)==1:
        tf=abs(np.fft.fft(signal))
        maxi=max(tf)
        mini=min(tf)
    
        #t=np.arange(len(tf))
        t2=np.arange(len(tf)//2)
        # -30 dB on smooth fft
        posi=np.argmax(np.abs(tf[t2]))
        maxi2=np.abs(tf[posi])
        seuil=maxi2/10**(30/20)
        lissage=100
        tfl=np.zeros(len(tf))
        tliss=np.arange(-lissage,lissage+1)
        for index in np.arange(lissage,len(tf)//2-lissage):
            tfl[index]=np.mean(np.abs(tf[index+tliss]))
        dem=np.where(tfl<seuil)[0]
        
        if len(dem)<2:
            fg=0
            fd=len(tf)//2-1
        else:
            fg=np.where((dem-posi)<0)[0]
            fd=np.where((dem-posi)>0)[0]
            fg=dem[fg[-1]]
            fd=dem[fd[0]]
            # Bigger band
            fen=fd-fg
            marge=0.25
            fg=np.floor(fg-fen*marge)/l
            fd=np.ceil (fd+fen*marge)/l
    
        if fdemod=='interactive':
            # interactive mode : ask the user for frequencies
            ff=plt.figure() 
            plt.plot(np.linspace(0,1,len(tf)),tf)
            plt.title("Draw a rectangle and clic on right button to validate")
            # plt.plot(np.linspace(0,1,len(tfl)),tfl)
            plt.xlabel ('Normalized Frequency')
            plt.ylabel ('Fourier transform')
            ax=plt.gca()
            ax.add_line(plt.Line2D([fg,fg],[mini,maxi],color='red',linestyle='--'))
            ax.add_line(plt.Line2D([fd,fd],[mini,maxi],color='red',linestyle='--'))
            ax.add_line(plt.Line2D([fd,fg],[mini,mini],color='red',linestyle='--'))
            ax.add_line(plt.Line2D([fd,fg],[maxi,maxi],color='red',linestyle='--'))

            def line_select_callback(eclick, erelease):
                if eclick.button==3:
                    rs.choiceok=1
 
            rs=wdg.RectangleSelector(ax,line_select_callback,drawtype='box', useblit=True,
                                       button=[1, 3],  # don't use middle button
                                       minspanx=5, minspany=5,
                                       spancoords='pixels',
                                       interactive=True)
            
            rs.choiceok=0
            
            #rs.set_visible(True)
            #rs.extents= 1 , 5 , -0.5, 0.5 
            #rs.to_draw=True
            #rs.update()
            
            plt.show()
            
            while rs.choiceok==0:
                plt.pause(0.3)
                x1,x2,y1,y2 = rs.extents
                
            if x1<0:
                x1=0
                
            if x2>0.5:
                x2=0.5

            fg=x1
            fd=x2

            #a=input ('Low frequency (suggested: '+str(fg)+' - enter to use it) ? :')
            #if a!='':
            #    fg=a

            #a=input ('High frequency (suggested: '+str(fd)+' - enter to use it) ? :')
            #if a!='':
            #    fd=a
            plt.close(ff)
    
        fdemod=[fg,fd]
    
    fdemodl=np.round(np.array(fdemod)*l)
    
    # Filter + analytic signal
    tf=np.fft.fft(signal)
    tf[0:int(fdemodl[0])]=0
    tf[int(fdemodl[1]):]=0
    sa=np.fft.ifft(tf)
    
    # Demodulation
    if method=='diff':
        phi=np.unwrap(np.angle(sa))
        freq=np.diff(phi)/(2*np.pi)
    elif method=='product':
        phi=np.unwrap(np.angle(sa))
        n=np.arange(1,len(signal)-1)
        freq[n]=(np.angle(-sa[n+1]*np.conj(sa[n-1]))+np.pi)/4/np.pi
        freq[0]=freq[1]
    else:
        raise ValueError('Illegal value for method')
    
    phase=phi
    freq=np.append(freq,freq[-1])
    
    return freq,phase,fdemod

# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("You should see a graph with a two plateau.")
    print("Draw a rectangle on the left one and right click.")
    print("Another graph with a ramp should appear.")
    t=np.arange(0,10,0.001)
    phi=2*np.pi*(50*t+2*t**2); sig=np.sin(phi)
    freq,phase,__=demodAnalytic(sig)
    plt.figure();
    plt.subplot(2,1,1); plt.plot(t,sig)
    plt.subplot(2,1,2); plt.plot(t,freq)
    


# -*- coding: utf-8 -*-
"""
Created on Sat Jul 27 09:14:48 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2019-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                  initial after the name)
"""

import sys # Requis pour PyQt5
from PyQt5 import QtCore, QtGui, QtWidgets

import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d
from matplotlib.backends.backend_qt5agg import (
    FigureCanvasQTAgg, NavigationToolbar2QT)

import numpy as np

class plotMatrix(QtWidgets.QMainWindow):
    """
    Draw an picture associated to a matrix and provide a toolbox to adjust visualisation.

    Example
    -------
    >>> import fbonnardot as fb; import numpy as np
    >>> data=np.array([[0,0,0,1,0],[1,2,1,0,0],[2,0,4,0,0],[3,0,0,6,0]])
    >>> fb.plotMatrix(data,0.1,2,'x','y','z')
    
    Warning
    -------
    Save your data before using this program : it can freeze Python
    """
    # Creation :    : Saturday 27 July 2019
    # Modifications : Monday 9 September 2019 (Use Qt instead of Tcl/Tk)
    #               : Tuesday 31 March 2020 (NumPy docstring - autotest update)
    # Version       : 1.2 i
    def __init__(self, data, stepx=1, stepy=1,xlabel='',ylabel='',zlabel=''):
        """
        Create the image properties window
        
        Parameters
        ----------
        
        data : matrix
            data to plot (y x x)
        
        stepx : integer, optional
            step between each elements on x axis
        
        stepy : integer, optional
            step between each elements on y axis
        
        xlabel : string, optional
            label for x axis
        
        ylabel : string, optional
            label for y axis
        
        zlabel : string, optional
            label for z axis
            
        Return
        ------
        
        Display a graph and a properties tuning window
        
        Example
        -------
        >>> import fbonnardot as fb; import numpy as np
        >>> data=np.array([[0,0,0,1,0],[1,2,1,0,0],[2,0,4,0,0],[3,0,0,6,0]])
        >>> fb.plotMatrix(data,0.1,2,'x','y','z')
        
        Warning
        -------
        Save your data before using this program : it can freeze Python
        """
        app = QtWidgets.QApplication.instance()
        if app is None:
            # Qt application does not run, so create it
            app = QtWidgets.QApplication([])   
            self.newapp=True
        else:
            # Qt application already running, so just show is sufficient
            self.newapp=False
            
        # Copy parameters to object instance
        self.data = data
        self.xlabel=xlabel
        self.ylabel=ylabel
        self.zlabel=zlabel
        # Creates x and y for plot 3D (add one line and column for pcolormesh)
        #   extra value is not used for other representations
        li,col=self.data.shape
        self.x=np.repeat([np.arange(col+1)],li+1,0)*stepx
        self.y=np.repeat(np.array([np.arange(li+1)]).T,col+1,1)*stepy
        # Creates the windows
        super(plotMatrix,self).__init__(None)
        self.makeWidgets()
        self.makeFigure2D()
        self.update()
        # Process events till end of program
        self.show()
        if self.newapp==True:
            # In case Qt application does not run
            app.exec_()

        
    def makeFigure(self,f3D):
        """Create a figure for display"""
        self.f3D=f3D
        # Creates the matplotlib figure
        self.fig=plt.Figure(figsize=(5, 4), dpi=100)
        if f3D:
            self.ax=self.fig.add_subplot(111,projection='3d')
        else:
            self.ax=self.fig.add_subplot(111)
        # Creates the dialog box with a Vertical Layout
        self.figdlg=QtWidgets.QDialog(self)
        self.figdlg.height=400
        self.figdlg.width=640
        self.vbl=QtWidgets.QVBoxLayout()
        self.figdlg.setLayout(self.vbl)
        # Creates Canvas i.e. a widget to display the figure in Qt5
        self.canvas = FigureCanvasQTAgg(self.fig)
        self.canvas.setSizePolicy(QtWidgets.QSizePolicy.Expanding,QtWidgets.QSizePolicy.Expanding)
        self.canvas.updateGeometry()
        # Creates a matplotlib toolbar
        self.toolbar = NavigationToolbar2QT(self.canvas,self.figdlg)
        self.toolbar.update()
        # Add these 2 widgets to vertical layout        
        self.vbl.addWidget(self.toolbar)
        self.vbl.addWidget(self.canvas)
        # Various actions
        self.canvas.draw()
        if f3D:
            self.ax.mouse_init()
        self.figdlg.show()
        
    def makeFigure2D(self):
        """Create a figure for 2D display"""
        self.makeFigure(False)

    def makeFigure3D(self):
        """Create a figure for 3D display"""
        self.makeFigure(True)
        
    def makeWidgets(self):
        """Setup windows widgets"""
        self.setObjectName("plotMatrix")
        self.setWindowTitle("plotMatrix")
        self.resize(296, 412)
        self.centralwidget = QtWidgets.QWidget(self)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        
        # Frame to choose the scale
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.label.setText("Scale")
        self.verticalLayout.addWidget(self.label)
        self.selectSc = QtWidgets.QComboBox(self.centralwidget)
        self.selectSc.setObjectName("selectSc")
        self.selectSc.addItem("Linear")
        self.selectSc.addItem("Logarithmic (20.log10(abs))")
        self.verticalLayout.addWidget(self.selectSc)

        # Frame to choose the colormap
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.label_2.setText("Colormap")
        self.verticalLayout.addWidget(self.label_2)
        self.selectCm = QtWidgets.QComboBox(self.centralwidget)
        self.selectCm.setObjectName("selectCm")
        self.selectCm.addItem("plasma")
        self.selectCm.addItem("inferno")
        self.selectCm.addItem("Greys")
        self.selectCm.addItem("gray")
        self.selectCm.addItem("hsv")
        self.selectCm.addItem("jet")
        self.verticalLayout.addWidget(self.selectCm)

        # Frame to choose the type of representation
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.label_3.setText("Representation")
        self.verticalLayout.addWidget(self.label_3)
        self.selectRep = QtWidgets.QComboBox(self.centralwidget)
        self.selectRep.setObjectName("selectRep")
        self.selectRep.addItem("picture")
        self.selectRep.addItem("contours")
        self.selectRep.addItem("3D")
        self.selectRep.addItem("surface")
        self.verticalLayout.addWidget(self.selectRep)

        # Frame to choose thresholds
        self.label_4 = QtWidgets.QLabel(self.centralwidget)
        self.label_4.setAlignment(QtCore.Qt.AlignCenter)
        self.label_4.setObjectName("label_4")
        self.label_4.setText("Threshold")
        self.verticalLayout.addWidget(self.label_4)
        self.selectThL = QtWidgets.QSlider(self.centralwidget)
        self.selectThL.setOrientation(QtCore.Qt.Horizontal)
        self.selectThL.setObjectName("selectThL")
        self.verticalLayout.addWidget(self.selectThL)
        self.selectThH = QtWidgets.QSlider(self.centralwidget)
        self.selectThH.setOrientation(QtCore.Qt.Horizontal)
        self.selectThH.setObjectName("selectThH")
        self.verticalLayout.addWidget(self.selectThH)

        self.selectThL.setMinimum(0)
        self.selectThL.setMaximum(100)
        self.selectThL.setValue(0)
        
        self.selectThH.setMinimum(0)
        self.selectThH.setMaximum(100)
        self.selectThH.setValue(100)
        
        
        # Frame to choose min and max frequency
        self.label_5 = QtWidgets.QLabel(self.centralwidget)
        self.label_5.setAlignment(QtCore.Qt.AlignCenter)
        self.label_5.setObjectName("label_5")
        self.label_5.setText("Displayed Frequency area")
        self.verticalLayout.addWidget(self.label_5)
        self.selectdfaL = QtWidgets.QSlider(self.centralwidget)
        self.selectdfaL.setOrientation(QtCore.Qt.Horizontal)
        self.selectdfaL.setObjectName("selectdfaL")
        self.verticalLayout.addWidget(self.selectdfaL)
        self.selectdfaH = QtWidgets.QSlider(self.centralwidget)
        self.selectdfaH.setOrientation(QtCore.Qt.Horizontal)
        self.selectdfaH.setObjectName("selectdfaH")
        self.verticalLayout.addWidget(self.selectdfaH)


        self.selectdfaL.setMinimum(0)
        self.selectdfaL.setMaximum(100)
        self.selectdfaL.setValue(0)
        
        self.selectdfaH.setMinimum(0)
        self.selectdfaH.setMaximum(100)
        self.selectdfaH.setValue(100)



        # frame with Button to update, create a new figure or quit
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btUp = QtWidgets.QPushButton(self.centralwidget)
        self.btUp.setObjectName("btUp")
        self.btUp.setText("Update Fig")
        self.horizontalLayout.addWidget(self.btUp)
        self.btNew = QtWidgets.QPushButton(self.centralwidget)
        self.btNew.setObjectName("btNew")
        self.btNew.setText("New Fig")
        self.horizontalLayout.addWidget(self.btNew)
        self.btQuit = QtWidgets.QPushButton(self.centralwidget)
        self.btQuit.setObjectName("btQuit")
        self.btQuit.setText("Quit")
        self.horizontalLayout.addWidget(self.btQuit)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.setCentralWidget(self.centralwidget)

        #self.btQuit.clicked.connect(self.close)
        QtCore.QMetaObject.connectSlotsByName(self)
        
        
    def update(self):
        """Remove old plot and plot according new parameters"""
        # Read values choosen by user in dialog box
        sc=self.selectSc.currentIndex()
        cm=self.selectCm.currentText()
        rp=self.selectRep.currentIndex()
        thL=self.selectThL.value()
        thH=self.selectThH.value()
        dfaL=self.selectdfaL.value()
        dfaH=self.selectdfaH.value()
        self.ax.clear()
        
        # Change figure type if necessary (2D or 3D)
        if not self.f3D and rp>=2:
            self.figdlg.destroy()
            self.makeFigure3D()
            
        if self.f3D and rp<2:
            self.figdlg.destroy()
            self.makeFigure2D()    
            
        # Create a copy of data to apply threshold and log scale if asked
        data=np.array(self.data)
        
        mini=np.min(self.data)
        maxi=np.max(self.data)
        thLv=mini+(maxi-mini)*thL/100
        thHv=mini+(maxi-mini)*thH/100
        data[data<thLv]=thLv
        data[data>thHv]=thHv
        
        # Compute the frequency area to display
        ylimmin=self.y[0][0]+(self.y[-1][0]-self.y[0][0])*dfaL/100
        ylimmax=self.y[0][0]+(self.y[-1][0]-self.y[0][0])*dfaH/100
        
        if sc==1:
            data=20*np.log10(np.abs(data))
        
        # Display data
        if rp==0:
            self.ax.pcolormesh(self.x,self.y,data,cmap=cm)
            self.ax.set_ylim(bottom=ylimmin,top=ylimmax)
        elif rp==1:
            self.ax.contour(self.x[0:-1,0:-1],self.y[0:-1,0:-1],data,cmap=cm)
            self.ax.set_ylim(bottom=ylimmin,top=ylimmax)
        elif rp==2:
            self.ax.plot_wireframe(self.x[0:-1,0:-1],self.y[0:-1,0:-1],data)
            self.ax.set_ylim(bottom=ylimmin,top=ylimmax)
        elif rp==3:
            self.ax.plot_surface(self.x[0:-1,0:-1],self.y[0:-1,0:-1],data,cmap=cm)
            self.ax.set_ylim(bottom=ylimmin,top=ylimmax)

        # Give title to axes
        self.ax.set_xlabel(self.xlabel)
        self.ax.set_ylabel(self.ylabel)
        if rp>=2:
            self.ax.set_zlabel(self.zlabel)
        
        # Ask to draw the figure contents
        self.canvas.draw()
        
    @QtCore.pyqtSlot()
    def on_btUp_clicked(self):
        self.update()

    @QtCore.pyqtSlot()
    def on_btNew_clicked(self):
        """Makes a new figure"""
        self.makeFigure2D()
        self.update()

    @QtCore.pyqtSlot()
    def on_btQuit_clicked(self):
        """Close all the windows to ends the application"""    
        self.close()
        if self.newapp==True:
            # Close application if it was not previously running
            QtWidgets.QApplication.closeAllWindows()
            QtWidgets.QApplication.exit()
        
    def closed(self):
        """Reacts to the close button on window bar (do not work everytime)"""
        #super().closed()
        if self.newapp==True:
            # Close application if it was not previously running
            QtWidgets.QApplication.closeAllWindows()
            QtWidgets.QApplication.exit()


# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("You should see a dialog box to adjust options, and a graph.")
    pm=plotMatrix(np.array([[0,0,0,1,0],[1,2,1,0,0],[2,0,4,0,0],[3,0,0,6,0]]),0.1,2,'x','y','z')
    pm.show()
    
    






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
fbonnardot: My functions (Frédéric BONNARDOT)
=============================================

Contents
--------
All fonctions of Frédéric BONNARDOT classified into thematic subpackages:

Subpackages
-----------
Importing fbonnardot imports all subpackages.

cyclostationarity
    Cyclostationary analysis of signal
detection
    Detection of peaks, ... on a signal
display
    Display utility (superposition, split into blocks, ...)
filtering
    Filtering fonctions (ekf, ...)
gui
    Various gui utilities
identification
    Identification of a filter, transfert function, ...
imagproc
    General image processing methods.
interpolation
    Interpolation methods
mechanic
    Functions to deal with gearbox, bearings, ...
sampling
    Resampling against angle, work with irregular sampling...
signalproc
    General signal processing methods
simulation
    Test signals
speed
    Estimates instantaneous speed and/or phasis
various
    Other types of functions

Note
----
Version 2020.04 01-Apr-2020
Copyright (c) 2001-2020 Frédéric BONNARDOT.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Warning
-------

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim damages or other liability.

"""

__author__ = "Frédéric BONNARDOT"
__copyright__ = "Copyright 2020, Frédéric BONNARDOT"
__credits__ = "Frédéric BONNARDOT"
__license__ = "AGPL-3.0-or-later license"
__version__ = "2020.04c"
__maintainer__ = __author__
__email__ = "frederic.bonnardot@univ-st-etienne.fr"
__status__ = "Prototype"

from .cyclostationarity import *
from .detection import *
from .display import *
#from .filtering import * 
#from .gui import *
#from .identification import *
#from .imageproc import *
#from .interpolation import *
#from .mechanic import *
#from .sampling import *
from .signalproc import *
#from .simulation import *
from .speed import *
#from .various import *



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
=================
Cyclostationarity
=================

Contents
--------
Cyclostationary analysis of signal:

Details
-------

cyclicTimeCorr
    Compute temporal (inter)-correlation of a cyclostationnary signal.
syncAv
    Computes the synchronous average
Note
----
Version 2019.11 09-Dec-2019
Copyright (c) 2001-2019 Frédéric BONNARDOT.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
Warning
-------

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim damages or other liability.

"""

__author__ = "Frédéric BONNARDOT"
__copyright__ = "Copyright 2019, Frédéric BONNARDOT"
__credits__ = "Frédéric BONNARDOT"
__license__ = "AGPL-3.0-or-later license"
__version__ = "2019.12"
__maintainer__ = __author__
__email__ = "frederic.bonnardot@univ-st-etienne.fr"
__status__ = "Prototype"

__all__=[
        'cyclicTimeCorr',
        'syncAv'
]

from .cyclicTimeCorr import cyclicTimeCorr
from .syncAv         import syncAv



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Dec 23 14:02:10 2018

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2001-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                     initial after the name)
"""


import math                              # for math.inf
import numpy as np                       # matrix management
import warnings
import matplotlib.pyplot as plt          # Plot
import matplotlib.backend_tools as tools # To create custom toolbar

plt.rcParams['toolbar'] = 'toolmanager'  # Pour ajouter une toolbar

def supPlot(datas,N=math.inf,offset=0,Ts=1,names=None,orient='vert',overlap=0,scale='ind',axes=None,t_axis=-1,sig_axis=None,fmt=None):
    """
    Display a set of signal by overlapping them
    
    Parameters
    ----------
    
    datas : matrix or tensor (np.array)
        datas to plot 
          - (matrix) nb_signal x time
          - or 3D array for many signal per stacks realization x nb_signal x time
          
    N : integer, optionnal
        last sample to plot (1 means 1st sample) must be > 1
        optional, math.inf by default (i.e. plot all samples)
        
    offset : scalar or vector, optionnal
        offset
        according to its sign offset is interpreted differently
            - offset > 0 : Display the data beginning at the offset+1 sample
            - offset < 0 : Tells the sample number corresponding to abscissa 0
                in the x axis
            - if offset is a vector, it will be like
                [display sample0] (with no sign interpretation)
        optional, 0 by default
        
    Ts : scalar of vector, optionnal
        - Sampling period or time vector
        - or vector corresponding to the abscissas
        optional, 1 by default
        
    names : array of string, optionnal
        names of the datas
        the name of the signals is displayed on the left
        ex. : ['channel A','channel B']
        if there is more the 20 signals, some name will not be displayed
        optional, None by default
        
    orient : string, optionnal
        Orientation of the signals name
            - 'horiz' : horizontal
            - 'vert'  : vertical
            - 'no'    : no signal name displayed
        optionnal, 'vert' by default
        
    overlap : integer, optionnal
        - Overlapping factor between signals between 0 and 0.999
        - or -1  to superpose the signals
        optionnal, 0 by default
        
    scale : string, optionnal
        Scale of signals
            - 'ind' : each signal have its own scale to fit in the n, n+1 box
            - 'glo' : same scale for each signal
        optionnal, 'ind' by default
        
    axes : object, optionnal
        Axes to draw the datas or None to use current Axes
        optionnal, None by default
        
    t_axis : vector, optionnal
        Axis associated to time,
        optionnal, -1 by default
        
    sig_axis : integer, optionnal
        Axis associated with a signal (i.e. stacks)
        optional, None by default
        None means other dimension for matrixes or -2 for 3D arrays
        
    fmt : string or list of string, optionnal
        format for plotting curves (e.g. 'b-')
            - str if datas is a matrix
            - or list of str if datas is 3D array, optional
        None by default
               
    Returns
    -------
    M         :number of signals
    
    Notes
    -----
    - the ith signal is between i and i+1 (1st signal i=1)
    - if overlap<>0, the min, max, the grid aren't displayed.
    - in case of 3D array, averaging against first axis for welch like Fourier
    
    Examples
    --------
    
    >>> import fbonnardot as fb; import numpy as np
    
    >>> datas=np.array([[1.,0.,0.,-1.,0.,1.],[2.,1.,0.,2.,0.,3.],[0.,0.,1.,-1.,1.,0.]])
    >>> fb.supPlot(datas,Ts=0.1,names=['sig 1','sig 2','sig 3'])

    >>> datas=np.array([[1.,0.,0.,-1.,0.,1.],[2.,1.,0.,2.,0.,3.],[0.,0.,1.,-1.,1.,0.]])
    >>> datasn=np.tile(datas,(4,1,1))+np.random.randn(4,3,6)
    >>> fb.supPlot(datasn,Ts=0.1,names=['sig 1','sig 2','sig 3'])
    
    In order to test FFT in the Toolbar:
    >>> t=np.arange(0,0.05,0.001)
    >>> datas=np.array([np.cos(2*3.14*50*t),np.cos(2*3.14*150*t),np.cos(2*3.14*200*t)])
    >>> fb.supPlot(datas,Ts=0.001,names=['50 Hz','150 Hz','200 Hz'])
    
"""    

    # Creation              : Thursay 12 July 2001      (MATLAB version)
    # Modifications         : Sunday 23 December 2018 (Translation to Python 3.6)
    #                         Monday 9 December 2019 (Add axes parameter)
    #                         Thurday 26 December 2019 (Can use 3D Matrixes)
    #                         Thursday 6 February 2020 (orient='no' option)
    #                         Tuesday 31 March 2020 (NumPy docstring - autotest)
    # Version               : 1.92 i


    # Find position of samples and signal axis in matrix datas
    mat_index=np.arange(datas.ndim)
    p_samples=mat_index[t_axis]
    if sig_axis==None:
        if datas.ndim==2:
            p_sig=1-p_samples
        else:
            p_sig=-2
    else:
        p_sig=mat_index[sig_axis]
    
    nb_sig=datas.shape[p_sig]
    t_sig=datas.shape[p_samples]
    #[nb_sig,t_sig]=datas.shape

    # Check that p_sig if different of t_sig
    if p_sig==p_samples:
        raise ValueError('Parameter t_axis should be different of sig_axis')
    
    # Check that signal length greater than number of signal
    if nb_sig>t_sig:
        warnings.warn('Number of signals to display greater than their length.\nYou may have forgot to transpose datas parameter, press Ctrl-C to stop.')
        
    # Get current axes if axes is None
    if axes==None:
        axes=plt.gca()
        menu=True
    else:
        menu=False
        
    # Manage optional parameters ...
    # ... Parameter N
    if N==math.inf:
        N=t_sig
    elif  N>t_sig:
        raise ValueError('Parameter N cannot be greater than signal length.')
    elif N<2:
        raise ValueError('Parameter N cannot be lower than 2.')
    
    if type(N) is not int:
        raise ValueError('N should be an integer.')
    
    # ... Parameter offset
    if np.isscalar(offset) and type(offset) is not int:
        raise ValueError('offset should be an integer.')
    
    if np.isscalar(offset):
        if offset>=N-1:
            raise ValueError('offset cannot be greater than N-2')
        elif offset>=0:
            firstSample=offset
            initialAbscissa=0
        else:
            firstSample=0
            initialAbscissa=-offset-1
    elif len(offset)==2:
        firstSample=offset[0]
        initialAbscissa=offset [1]-1
    else:
        raise ValueError('Offset can\'t have more than 2 elements.')
    
    # ... Ts
    if not np.isscalar(Ts):
        if (len(Ts) != t_sig):
            raise ValueError('Ts must have the same size than each signals.')
    
    # ... noms
    if names!=None:
        if len(names)!=nb_sig:
            raise ValueError ('The last signals don\'t have names.')
                              
    # ... orientation
    if orient!='vert' and orient!='horiz' and orient!='no':
        raise ValueError('Bad value for orientation.')
    
    # ... overlap
    if (abs(overlap)>=1 or abs(overlap)<0) and overlap!=-1:
        raise ValueError('Overlap must be in [0;1[.')
    
    # ... scale
    if scale=='glo':
        sameScale=1
    elif scale=='ind':
        sameScale=0
    else:
        raise ValueError('Bad value for scale.')
    
    if overlap==-1:
        sameScale=1
    
    #...fmt
    if fmt==None:
        fmt='b'
        
    if datas.ndim==3 and np.ndim(fmt)==0:
        fmt=np.tile(fmt,datas.shape[0])
    
    # Extract data to display (work on a copy of datas)
    temp=np.array(datas)
    temp=np.swapaxes(temp,p_samples,0)
    if p_sig==0:
        p_sig=p_samples
    if datas.ndim==3:
        temp=np.swapaxes(temp,p_sig,1)
    # Now temp size is nb_samples x nb_sig x nb_realizations
    datas2=np.array(temp.T)
    temp=temp[firstSample:N,:]
    # if p_sig==0:
    #     temp=np.array(datas[:,firstSample:N].T)
    # else:
    #     temp=np.array(datas[firstSample:N,:])
        
    # Signal normalisation : they should be in index*(1-overlap)+[-0.5;0.5]
    if datas.ndim==2:
        maximum=np.max (temp,0)
        minimum=np.min (temp,0)
    else:
        maximum=np.max (temp,(0,2))
        minimum=np.min (temp,(0,2))
                
    if sameScale==1:
        minimum[:]=np.min(minimum)
        maximum[:]=np.max(maximum)
    
    if overlap!=-1:
        for index in range(nb_sig):
            if maximum[index]==minimum[index]:
                warnings.warn('The signal n°'+str(index)+' is constant !!!')
                temp[:,index]=(index+1)*(1-overlap)+0.5
            else:
                temp[:,index]=(index+1)*(1-overlap)+0.5+(temp[:,index]-(maximum[index]+minimum[index])/2)/(maximum[index]-minimum[index])
    
    # Create or extract time vector
    if np.isscalar(Ts):
        t=np.arange((firstSample-initialAbscissa),(N-initialAbscissa))*Ts
    else:
        t=Ts[firstSample:N]
        
    # Plots the data
    if datas.ndim==2:
        axes.plot(t,temp,fmt)
    else:
        for index in range(temp.shape[-1]):
            axes.plot(t,temp[:,:,index],fmt[index])

    # Choose area to display
    axes.set_xlim(t[0],t[-1])
    if overlap!=-1:   
        axes.set_ylim(1-overlap,nb_sig*(1-overlap)+1)
    else:
        axes.set_ylim(min(temp),max(temp))
        names=None
    
    
    # Display a grid if no overlap
    if overlap==0:
        # Limit number of grid line if more than 20 signals
        if nb_sig>20:
            pastick=math.ceil (nb_sig/20)
        else:
            pastick=1
        axes.set_yticks(np.arange(2,nb_sig+1,pastick),[])
        axes.grid()
    
    # Display names
    if names!=None:
        #set (handle,'YTickLabel',[]);
        # Limit the number of name to display if more than 20 signals
        if nb_sig>20:
            namestep=math.ceil (nb_sig/20)
        else:
            namestep=1
        
        for index in np.arange(0,nb_sig,namestep):
            name=names[index]
            if orient=='vert':
                axes.text(t[0],(index+1)*(1-overlap)+0.5,name,rotation=90,ha='right',va='center')
            elif orient=='horiz':
                axes.text(t[0],(index+1)*(1-overlap)+0.5,name,ha='right',va='center')
    
    # Display minimum and maximum values if no overlap
    if overlap==0 and sameScale==0 and nb_sig<20:
        if datas.ndim==2:
            for index in range(nb_sig):
                text="{:01.2e}".format(np.min(datas[index,firstSample:N]))
                axes.text(t[-1],index+1,text,ha='left',va='bottom')
                text="{:01.2e}".format(np.max(datas[index,firstSample:N]))
                axes.text(t[-1],index+2,text,ha='left',va='top')
        else:
            for index in range(nb_sig):
                text="{:01.2e}".format(np.min(datas[:,index,firstSample:N]))
                axes.text(t[-1],index+1,text,ha='left',va='bottom')
                text="{:01.2e}".format(np.max(datas[:,index,firstSample:N]))
                axes.text(t[-1],index+2,text,ha='left',va='top')  


    # Classes associated to the toolbar

    class StretchTool(tools.ToolBase):
        """Switch between global and individual scale"""
        default_keymap = 'S'
        description = 'Stretch individualy each signal or apply the same global scale'
        # image = None
        name = 'Stretch'
    
        def __init__(self, *args, effect, **kwargs):
            self.name=effect
            super().__init__(*args, **kwargs)
    
        def trigger(self, *args, **kwargs):
            spparams=self.figure.spparams
            plt.figure()
            if spparams['scale']=='glo':
                scale='ind'
            else:
                scale='glo'
            supPlot(spparams['datas'],spparams['N'],spparams['offset'],spparams['Ts'],spparams['names'],spparams['orient'],spparams['overlap'],scale,fmt=fmt)

    class FourierTool(tools.ToolBase):
        """Display Fourier Transform modulus (Hamming Window)"""
        default_keymap = 'F'
        description = 'Display the Fourier Transform modulus of the signals (Hamming Window)'
        # image = None
        name = 'FFT'
      
        def __init__(self, *args,unit, **kwargs):
            self.name='FFT('+unit+')'
            self.unit=unit
            super().__init__(*args, **kwargs)

        def trigger(self, *args, **kwargs):
            spparams=self.figure.spparams
            # Compute Fourier Transform of each data
            datas=spparams['datas']            
            [nb_sig,t_sig]=datas.shape[-2:]
            datasft=np.zeros([nb_sig,t_sig])
            window=np.hamming(t_sig)
            for index in range(nb_sig):
                if datas.ndim==2:
                    datasft[index,:]=np.abs(np.fft.fft(datas[index,:]*window))
                else:
                    datasft[index,:]=np.mean(np.abs(np.fft.fft(datas[:,index,:]*window,axis=1)),axis=0)
            if self.unit=='dB':
                datasft=20*np.log10(datasft)
            plt.figure()
            supPlot(datasft,math.inf,0,1/spparams['Ts']/t_sig,spparams['names'],spparams['orient'],spparams['overlap'],spparams['scale'])


    class CepstrumTool(tools.ToolBase):
        """Display Real Cepstrum"""
        default_keymap = 'C'
        description = 'Display the Real Cepstrum of the signals'
        # image = None
        name = 'Cepstrum'

        def trigger(self, *args, **kwargs):
            spparams=self.figure.spparams
            # Compute Fourier Transform of each data
            datas=spparams['datas']            
            [nb_sig,t_sig]=datas.shape[-2:]
            datasrceps=np.zeros([nb_sig,t_sig])
            for index in range(nb_sig):
                if datas.ndim==2:
                    datasrceps[index,:]=np.real(np.fft.ifft(np.log(np.abs(np.fft.fft(datas[index,:])))))
                else:
                    datasrceps[index,:]=np.real(np.fft.ifft(np.log(np.mean(np.abs(np.fft.fft(datas[:,index,:],axis=1)),axis=0))))
            plt.figure()
            supPlot(datasrceps,math.inf,0,spparams['Ts'],spparams['names'],spparams['orient'],spparams['overlap'],spparams['scale'])


    class OverlapTool(tools.ToolBase):
        """Overlap all signals"""
        default_keymap = 'O'
        description = 'Overlap all signals'
        # image = None
        name = 'Overlap'

        def trigger(self, *args, **kwargs):
            spparams=self.figure.spparams
            datas=spparams['datas']
            [nb_sig,t_sig]=datas.shape[-2:]
            t=np.arange(t_sig)/spparams['Ts']
            plt.figure()
            if datas.ndim==2:
                plt.plot(t,datas.T)
            else:
                for index in range(datas.shape[0]):
                    plt.plot(t,datas[index].T)

    # User menu (if no axes is given)
    if menu==True:
        # Store the params in the figure
        spparams={"datas":datas2,"N":N,"offset":offset,"Ts":Ts,"names":names,"orient":orient,"overlap":overlap,"scale":scale,"fmt":fmt}
        fig=plt.gcf()
        fig.spparams=spparams
    
        # Set up the toolbar
        if scale=='glo':
            effect='Stretch'
        else:
            effect='UnStretch'
        fig.canvas.manager.toolmanager.add_tool(effect, StretchTool,effect=effect)
        fig.canvas.manager.toolmanager.add_tool('Overlap', OverlapTool)
        fig.canvas.manager.toolmanager.add_tool('FFT(lin)', FourierTool,unit='lin')
        fig.canvas.manager.toolmanager.add_tool('FFT(dB)', FourierTool,unit='dB')
        fig.canvas.manager.toolmanager.add_tool('Cepstrum', CepstrumTool)
            
        fig.canvas.manager.toolbar.add_tool(effect,    'supplot', 0)
        fig.canvas.manager.toolbar.add_tool('Overlap', 'supplot', -1)
        fig.canvas.manager.toolbar.add_tool('FFT(lin)','supplot', -1)
        fig.canvas.manager.toolbar.add_tool('FFT(dB)', 'supplot', -1)
        fig.canvas.manager.toolbar.add_tool('Cepstrum','supplot', -1)
    
    # Missing in comparison with Matlab version :
    #    psd lin and log with block size of 256/512/1024/2084/4096
    #    timefrequency
    #    extract a signal
    #    remove a signal
    #    overlap ((0:19)*5 %)
 
    return nb_sig

# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("You should see a dialog box to adjust options, and a graph.")
    datas=np.array([[1.,0.,0.,-1.,0.,1.],[2.,1.,0.,2.,0.,3.],[0.,0.,1.,-1.,1.,0.]])
    supPlot(datas,Ts=0.1,names=['sig 1','sig 2','sig 3'])
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb  1 10:44:43 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2003-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                       initial after the name)
"""

import numpy             as np                 # matrix management
import scipy.interpolate as interp             # interpolation

def peakDetection (signal,method='kStdThreshold',k=3,Nmax=np.Inf,k2=None):
    """
    Detect peaks in a signal
    
    Parameters
    ----------
    
    signal : vector
        signal
        
    method : string, optionnal
        - 'kStdThreshold' : use a threshold=k*std(signal) and return the Nmax
            biggest values
        - 'diff' : differenciate the signal and look at sign changes (+ to -)
        - 'diffInterp' : differenciate the signal and look at sign changes (+ to -)
            use interpolation to find accurate results
        optionnal, by default method='kStdThreshold'
 
    k : scalar, optionnal
        meaning depends of the method used
            - 'kStdThreshold' : coefficient for selection
            - 'diff'          : minimum spacing between peaks
        optionnal, k=3 by default
                                   
    Nmax : integer, optionnal
        maximum number of peaks to be detected
        optionnal, Nmax=Inf by default
 
    k2 : integer, optionnal
        - for 'kStdThreshold' : minimum spacing between peaks,
            optionnal, k=1 by default
        - for 'diffinterp' : interpolation factor
            optionnal, 4 by default
 
    Return
    ------
    position : vector
        peaks position
        
    amp : vector
        amplitude of signal signal at peaks position
        
    Example
    -------
    >>> import fbonnardot as fb; import matplotlib.pyplot as plt
    >>> import scipy.signal as sigp; import numpy as np
    >>>
    >>> # Generate test signal
    >>> timp=np.array(np.cumsum(np.random.rand(30)*10+1000),int)-500
    >>> signal=np.zeros(int(max(timp))+500)
    >>> signal[timp]=3+np.random.randn(len(timp))*0.2
    >>> signal=sigp.lfilter([1],[1,0.9],signal); signal+=np.random.randn(len(signal))*0.03
    >>> #plt.figure(); plt.plot(signal)
    >>> signal=sigp.decimate(signal,10)
    >>> b,a=sigp.butter(3,2*6.28/100)
    >>> signal2=sigp.filtfilt(b,a,signal)
    >>>
    >>> positionts1,ampts1=fb.peakDetection(signal,'kStdThreshold')
    >>> positionds1,ampds1=fb.peakDetection(signal,'diff',Nmax=40)
    >>> positionts2,ampts2=fb.peakDetection(signal2,'kStdThreshold')
    >>> positionds2,ampds2=fb.peakDetection(signal2,'diff',Nmax=40)
    >>>
    >>> # Graphs
    >>> plt.figure()
    >>> plt.subplot(2,1,1)
    >>> plt.plot(signal)
    >>> plt.plot(positionds1,ampds1,'o')
    >>> plt.plot(positionts1,ampts1,'x')
    >>> plt.subplot(2,1,2)
    >>> plt.plot(signal2)
    >>> plt.plot(positionds2,ampds2,'o')
    >>> plt.plot(positionts2,ampts2,'x')
    >>> plt.legend(['signal','diff method','threshold method'])


    Note
    ----
    * NaN and Inf are not considered as peaks
    * Outputs vector are sorted by amplitude descending order
    """

    # Creation              : Monday 26 May 2003 (MATLAB version)
    # Modifications         : Friday 1st February 2019 (Translation to Python 3.6)
    #                         Thursay 12 December 2019 (Correct bug in peakSelection)
    #                         Tuesday 17 December 2019 (Change algorithm for peakSelection)
    #                                                  (Add progress bar)
    #                         Wednesday 1st April 2020 (Auto-test, example and Numpy Docstrings)
    # Version               : 1.2 i

    # Check arguments
    if method not in ('kStdThreshold','diff','diffInterp'):
        raise ValueError('Illegal value for method.')
    
    if k2==None:
        if method=='kstdthreshold':
            k2=1
        else:
            k2=4
    
    # Remove NaN and Inf
    signal=np.array(signal) # Work on a copy of signal
    ok=np.where(np.isfinite(signal))[0]
    pb=np.where(np.isfinite(signal)==False)[0]
    signal[pb]=np.min(signal[ok])
    
    # Choose method
    if method=='kStdThreshold':
        return kStdThreshold (signal,k,Nmax,k2)
    elif method=='diff':
        return diffmethod    (signal,k,Nmax)
    elif method=='diffInterp':
        return diffinterp    (signal,k,Nmax,k2)       

    
def kStdThreshold (signal,k,Nmax,mindist):
    
    # Detection of low probabilistic values according to a Gaussian low
    m=np.mean(signal)
    signal=signal-m
    threshold=np.std(signal)*k
    pos=np.where(signal>threshold)[0]
    amp=signal[pos]
    
    # Sort this values by descending order
    #[order,permutation]=sort(amp.*-1);
    #pos=pos(permutation);
    
    # Limitation to Nmax values
    #if length(position)>Nmax
    #    position=position(1:Nmax);
    #end;
    position=peakSelection (pos,amp,Nmax,mindist)
    
    amp=signal[position]+m
    
    return position,amp
    

    
def diffmethod (signal,mindist,Nmax):
    
    if mindist<=0:
        raise ValueError('The minimum distance beetween peaks cannot be 0 or negative.')
    
    N=len(signal)
    m=min(signal)
    
    # Find peak location and value by differenciation
    deriv=np.diff(np.concatenate(([m] , signal , [m])))
    pos=np.where(np.logical_and(deriv[0:N]>=0,deriv[1:N+1]<=0))[0]
    amp=signal[pos]
    
    # Peak selection
    position=peakSelection (pos,amp,Nmax,mindist)
    amp=signal[position]
    
    return position,amp
    
    
def diffinterp (signal,mindist,Nmax,ifactor):
    
    #%signal=resample (signal,ifactor,1);
    #%[position,amp]=diffmethod (signal,mindist,Nmax);
    #%position=position./ifactor;
    
    # Use classic method to find position
    position,amp=diffmethod (signal,mindist,Nmax)
    
    N=len (signal)
    
    # Refine each position
    delta=np.arange(-10,11)
    for index in range(len(position)):
        pos=position[index]
        if pos>4 and pos<(N-6):
            fx=interp.interp1d(delta,signal[pos+delta])
            y=fx(np.arange(-ifactor,ifactor+1)/ifactor)
            delta=np.argmax(y)
            position[index]=pos+(delta-ifactor-1)/ifactor
            
    return position,amp



def peakSelection (pos,amp,Nmax,mindist):
    """
    Selection Nmax higher peaks from the list and ignore close peaks (close mean <mindist)
    
    Input
    -----
    pos     : position of the peaks
    amp     : amplitude of the peaks
    Nmax    : maximum number of peaks to select
    mindist : minimum distance between peaks
    
    Return
    ------
    pos     : selected peaks position
    """
    
    # Sort positions by decreasing amplitudes
    asrt=np.argsort(-amp)
    pos=pos[asrt]
    # Go from highest to lowest amplitude to remove close peaks
    n=0
    while n<Nmax and len(pos)>n:
        NN=np.min((Nmax,len(pos)))
        progressBar(n/NN)
        # Remove close peaks
        dist=np.abs(pos-(pos[n]))
        dist[n]=mindist+1 # To keep current at a dist=0
        farp=np.nonzero(dist>=mindist)[0] # remove when dist<mindist
        pos=pos[farp]
        # Go to next value
        n=n+1

    progressBar(-1)
    
    return np.sort(np.array(pos[0:n]))

def progressBar(position):
    """
    Display a text progress bar
    
    Parameters
    ----------
    
    position   : between 0 and 1
                 otherwise suppress the bar
    """
    
    if position>=0 and position<=1:
        nb_done=int(np.ceil(position*50))
        print("\rpeakDetection : \u001b[44;1m"+">"*nb_done+"\u001b[0m\u001b[44m"+"-"*(50-nb_done)+"\u001b[0m",end='')
    else:
        print("\r"+" "*70+"\r\u001b[A")
        
# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    import scipy.signal as sigp; import matplotlib.pyplot as plt
    print("Auto-test if Python script launched from console")
    print("You should see two sub figures.")
    print("Figure 1 : a noisy signal with peaks. Peak are marked with a cross and round.")
    print("Figure 2 : low pass filtered version with peak detection.")

    # Generate test signal
    timp=np.array(np.cumsum(np.random.rand(30)*10+1000),int)-500
    signal=np.zeros(int(max(timp))+500)
    signal[timp]=3+np.random.randn(len(timp))*0.2
    signal=sigp.lfilter([1],[1,0.9],signal); signal+=np.random.randn(len(signal))*0.03
    #plt.figure(); plt.plot(signal)
    signal=sigp.decimate(signal,10)
    b,a=sigp.butter(3,2*6.28/100)
    signal2=sigp.filtfilt(b,a,signal)
    
    positionts1,ampts1=peakDetection(signal,'kStdThreshold')
    positionds1,ampds1=peakDetection(signal,'diff',Nmax=40)
    positionts2,ampts2=peakDetection(signal2,'kStdThreshold')
    positionds2,ampds2=peakDetection(signal2,'diff',Nmax=40)
    
    # Graphs
    plt.figure()
    plt.subplot(2,1,1)
    plt.plot(signal)
    plt.plot(positionds1,ampds1,'o')
    plt.plot(positionts1,ampts1,'x')
    plt.subplot(2,1,2)
    plt.plot(signal2)
    plt.plot(positionds2,ampds2,'o')
    plt.plot(positionts2,ampts2,'x')
    plt.legend(['signal','diff method','threshold method'])
    


# -*- coding: utf-8 -*-
"""
Created on Tue Nov 19 17:15:04 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2017-2019

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                       initial after the name)
"""

import numpy as np                       # matrix management
import matplotlib.pyplot as plt          # plots
from mpl_toolkits.mplot3d import axes3d  # 3D projection
import fbonnardot.cyclostationarity.syncAv

def cyclicTimeCorr (x,y,period,vect_tau,graph=0):
    """
    Compute temporal (inter)-correlation of a cyclostationnary signal.

    Parameters
    ----------   
    x,y : vector,
        signals x and y must have the same size
        
    period : scalar,
        cyclic period of the signal to consider
        
    vect_tau : vector,
        delays (lag) for computing (even integers)

    graph : scalar, optionnal
        display a graph if 1
        optionnal, 0 by default
        
    Returns
    -------
    Rxy : matrix of size period x len(vect_tau)
        Estimated cyclic correlation Rxy(t,tau)
                      
    Note
    -----
    * To estimate correlation, we compute
        syncAv(x(t-tau/2)*conj(y(t+tau/2)),period)

    * Since synchronous average is used, all cyclic period not multiple of period
        are destroyed or reduced

    Example
    -------
    >>> import fbonnardot as fb
    >>> import scipy.signal as signal; import numpy as np
    >>> per=12; N=130*per
    >>> bruit=np.random.rand(N)-0.5
    >>> bruit=signal.lfilter([1,0.9,0.8,0.7,0.4,0.5,0.2,0.1],1,bruit) # Correlate noise
    >>> data=np.sin(2*np.pi/per*np.arange(N))*bruit
    >>> tau=np.arange(0,20,2)
    >>> Rxx=fb.cyclicTimeCorr(data,data,per,tau,graph=1)
    """
    
    # Creation              : Tuesday 4 July 2017      (MATLAB version)
    # Modifications         : Tuesday 19 November 2019 (Translation to Python 3.7)
    #                         Thursday 2 April 2020    (Docstring anf Auto test)
    # Version               : 1.1 i

    Nx=len(x)
    Ntau=len(vect_tau)

    if Nx!=len(y):
        raise ValueError ("x and y must have the same size")

    Rxy=np.zeros((period,Ntau))


    for index in range(Ntau):
        progressBar(index/Ntau)
        tau=vect_tau[index]
        
        t=np.arange(tau//2+1,Nx-tau//2)
        t=t[0:int(np.floor(len(t)/period))*period] # integer number of period
        sAv,nb=fbonnardot.cyclostationarity.syncAv(x[t-tau//2]*np.conj(y[t+tau//2]),period)
        Rxy[:,index]=sAv

    progressBar(-1)
    
    # Graph if graph=1
    if graph==1:
        fig = plt.figure()
        t=np.arange(0,period)
        ax = fig.add_subplot(111, projection='3d')
        for index in range(Ntau):
            ax.plot(vect_tau[index]*np.ones(len(t)),t,np.abs(Rxy[:,index]))
        ax.set_xlabel(r'$\tau$'); ax.set_ylabel(r'$t$'); ax.set_zlabel(r'$|Rxy(t,\tau)|$');
        ax.view_init(elev=30, azim=60)
        plt.title('Modulus of Time Correlation')
        plt.show()
    
    return Rxy


def progressBar(position):
    """
    Display a text progress bar
    
    Parameters
    ----------
    
    position   : between 0 and 1
                 otherwise suppress the bar
    """
    
    if position>=0 and position<=1:
        nb_done=int(np.ceil(position*50))
        print("\rcyclicTimeCorr : \u001b[44;1m"+">"*nb_done+"\u001b[0m\u001b[44m"+"-"*(50-nb_done)+"\u001b[0m",end='')
    else:
        print("\r"+" "*70+"\r\u001b[A")
        
# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("A 3D figure is displayed.")
    print("At lag tau=0 there is 2 large peaks.")
    print("The amplitude of these peaks vanished as tau increase.")
    
    import scipy.signal as signal
    per=12; N=130*per
    bruit=np.random.rand(N)-0.5
    bruit=signal.lfilter([1,0.9,0.8,0.7,0.4,0.5,0.2,0.1],1,bruit) # Correlate noise
    data=np.sin(2*np.pi/per*np.arange(N))*bruit
    tau=np.arange(0,20,2)
    Rxx=cyclicTimeCorr(data,data,per,tau,graph=1)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jan  7 22:33:24 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2001-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                       initial after the name)
"""

import numpy as np                       # matrix management
import scipy.interpolate as interp       # interpolation
import warnings

def syncAv (datas,blocSize,resid=False):
    """
    Computes the synchronous average of one or more signals.
    
    Parameters
    ----------
    datas : np.array vector or matrix (nb_sig x sig_len) 1 signal=1 row
        signals used for compute synchronous average 
        
    blocSize : int or float
        size of a block to make the mean (not necessary an integer)
        
    resid : boolean, optional
        returns the residual if resid is True
        optional, False by default

    Returns
    -------
    sav : np.array
        Synchronous average of each signals
        
    nbBlocs : int
        Number of blocs used for estimating the synchronous average
        if this size is not integer, the data will be interpolated to make this size integer

    residual : np.array, optional
        residual returned only if resid is True

    Note
    ----
    * A warning is made if the number of row is greater than the number of column.
    * A warning is made if there is not an integer number of cycles
    
    Examples
    --------
    
    Case were period is an integer
    
    >>> import fbonnardot as fb
    >>> from scipy import signal
    >>> import matplotlib.pyplot as plt
    >>> import numpy as np
    >>>
    >>> period=128; cycles=100
    >>> zz=np.zeros((period-1)//2)
    >>> exitation=np.tile(np.concatenate([zz,[0,1],zz]),[cycles])
    >>> sig=signal.lfilter([1],[1,0.3,-0.2,0.4,0.6],exitation)
    >>> sig=sig+np.random.randn(period*cycles)*0.1
    >>>
    >>> sav,nbBlocs=fb.syncAv (sig,period)
    >>> plt.figure()
    >>> plt.plot(sig.T,'b')
    >>> plt.plot(sav.T,'r')
    >>> plt.legend(['signal (1 period)','synchronous average'])
    >>> plt.xlim([0,period])
    
    Case with an real period
        
    >>> import fbonnardot as fb
    >>> from scipy import signal
    >>> import matplotlib.pyplot as plt
    >>> import numpy as np    
    >>>
    >>> period=127.4; cycles=100
    >>> sig=np.sin(2*np.pi*np.arange(int(period*cycles))/period)
    >>> sig=sig+np.random.randn(np.size(sig))*0.1
    >>>
    >>> sav,nbBlocs=fb.syncAv (sig,period)
    >>> plt.figure()
    >>> plt.plot(sig,'b')
    >>> plt.plot(np.linspace(0,period,int(np.ceil(period))),sav,'r')
    >>> plt.legend(['signal (1 period)','synchronous average'])
    >>> plt.xlim([0,period])

    """

    # Creation              : Thursay 23 August 2001 (MATLAB version)
    # Modifications         : Monday 7 January 2019  (Translation to Python 3.6)
    #                         Saturday 9 March 2019   (Can return the residual)
    #                         Thursday 2 April 2020 (Auto test)
    # Version               : 1.4 i

    # Original name : msynchrone.m

    if np.ndim(datas)>1:
        li,col=np.shape (datas)
    else:
        col=np.size(datas)
        li=1
        
    if blocSize>col:
        raise ValueError ('The datas must have at least 1 blocs.')
    
    if blocSize<=0:
        raise ValueError ('Bloc must be strictly positive.')
    
    if li>=col:
        warnings.warn('Number of row>number of column ? 1 row = 1 signal')
   
    # Manage not integer periods
    if type(blocSize) is not int:
        warnings.warn ('Not an integer period => resampling.')
        step=blocSize/np.ceil(blocSize)
        if li>1:
            donnees2=np.zeros(li,int(np.ceil((col-1)/step)))
            for index in range(li):
                ifunc=interp.interp1d(np.arange(1,col+1),datas[index,:],kind='cubic')
                donnees2[index,:]=ifunc(np.arange(1,col,step))
        else:
            ifunc=interp.interp1d(np.arange(1,col+1),datas,kind='cubic')
            donnees2=ifunc(np.arange(1,col,step))
        if resid:
            sav,nbBlocs,res=syncAv (donnees2,int(np.ceil(blocSize)),resid)
            residu=res
        else:
            sav,nbBlocs=syncAv (donnees2,int(np.ceil(blocSize)),resid)
    else:
        # Compute synchronous average
        nbBlocs=col//blocSize
        
        if col % blocSize!=0:
            warnings.warn ('The signal don\'t contain an integer number of intergers, it will be truncated.')
        
        if nbBlocs==1:
            warnings.warn ('With only 1 bloc it is not synchronous average !!!')
        
        if li>1:
            # nbr sig x sig_len -> nbr_sig x nbBlocs x blocSize
            tens=np.reshape(datas[:,0:(nbBlocs*blocSize)],[li,nbBlocs,blocSize])
            # nbr_sig x nbBlocs x blocSize -> nbr_sig x blocSize
            sav=np.mean(tens,1)
            residu=datas-np.tile(sav,nbBlocs+1)[0:li,0:col]
        else:
            # sig_len -> nbBlocs x blocSize
            tens=np.reshape(datas[0:(nbBlocs*blocSize)],[nbBlocs,blocSize])
            sav=np.mean(tens,0)
            residu=datas-np.tile(sav,nbBlocs+1)[0:col]
            
    if resid:
        return sav,nbBlocs,residu
    else:
        return sav,nbBlocs

# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("Figure 1 shows in blue the first period of an impulsive noisy signal.")
    print("   Its synchronous average is in red.")
    print("Figure 2 shows in blue the first period of an sine noisy signal.")
    print("   Its synchronous average is in red.")
    
    from scipy import signal
    import matplotlib.pyplot as plt

    period=128; cycles=100
    zz=np.zeros((period-1)//2)
    exitation=np.tile(np.concatenate([zz,[0,1],zz]),[cycles])
    sig=signal.lfilter([1],[1,0.3,-0.2,0.4,0.6],exitation)
    sig=sig+np.random.randn(period*cycles)*0.1

    sav,nbBlocs=syncAv (sig,period)
    plt.figure()
    plt.plot(sig.T,'b')
    plt.plot(sav.T,'r')
    plt.legend(['signal (1 period)','synchronous average'])
    plt.xlim([0,period])
    
    # Case with an real period
    
    period=127.4; cycles=100
    sig=np.sin(2*np.pi*np.arange(int(period*cycles))/period)
    sig=sig+np.random.randn(np.size(sig))*0.1

    sav,nbBlocs=syncAv (sig,period)
    plt.figure()
    plt.plot(sig,'b')
    plt.plot(np.linspace(0,period,int(np.ceil(period))),sav,'r')
    plt.legend(['signal (1 period)','synchronous average'])
    plt.xlim([0,period])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jan 31 21:10:43 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2003-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                       initial after the name) 
"""

import numpy             as np                 # matrix management
import scipy.signal      as sigp               # signal processing
import scipy.interpolate as interp             # interpolation
import matplotlib.pyplot as plt                # plot functions
import fbonnardot.detection.peakDetection as peakDetection
import fbonnardot.signalproc.circShift as circShift
import fbonnardot.display.periodPlot as periodPlot
import fbonnardot.display.supPlot as supPlot

def synchronisation2 (signal,period,method='maxCxyint',param=None,scaleopt='none',estsync=True,compens=False,graph=0):
    """
    Synchronisation of data by using intercorrelation or an amplitude based
    detection.
    
    Parameters
    ----------
    signal : vector
        signal to synchronize
    
    period : vector or int
        average period of the signal or vector with the reference signal
                  
    method : str, optional
        method used for synchronization
            * 'maxCxy'     - maximum of cross-correlation
            * 'baryCxy'    - barycenter of cross-correlation
            * 'maxCxyint'  - like Cxy but with 10x interpolation on Cxy for more precision
            * 'threshold'  - 1st sample at amplitude param
            * 'rthreshold' - 1st sample with positive slope and amplitude greater or equal to threshold
            * 'max'        - position of the maximum amplitude
            * 'ceps'       - robust cepstrum
            
        optionnal, 'maxCxyint' by default
                
    param : float tuple or None, optional
        According to method
            * 'threshold'  : float correspondinf to threshold
            * 'thresholdr' : tupple (threshold,min_slope) 
            * else not used (None)
        
        optional, None by default
    
    scaleopt : str, optional
        parameter for the correlation fonction
            * 'none'
            * 'biased'
            * 'unbiased'
            * 'circ' : correlation between reference and repetead 2 times signal
            
        optional, 'none' by default
                
    estsync : bool, optional
        True to fill synchr, False to return synchr=None
        
        optional, True by default
                
    compens : bool, optional
        True for compensation of slipping by shifting the begining of a bloc if
        the period change too much
        
        optional, False by default
                
    graph : int, optional
        Show results on one more graph
            * if graph & 1 = 1 -> supPlot
            * if graph & 2 = 2 -> block superposition before, after
            * if graph & 4 = 4 -> blocks
            
        optional, 0 by default
 
    Return
    ------
    synchr : Matrix Ncycles x period or None
        synchronized signals  
    
    delta : vector
        value of shift compared to period, ith value is at i*period-delta[i]
 
    Note
    ----
    Difference with synchronisation : this method correct the last period
    variation before each estimation.
    
    Example
    -------
    >>> import fbonnardot as fb; import matplotlib.pyplot as plt
    >>> import scipy.signal as sigp; import numpy as np
    >>>
    >>> # Generate test signal
    >>> timp=np.array(np.cumsum(np.random.rand(30)*10+1000),int)-500
    >>> signal=np.zeros(int(max(timp))+500)
    >>> signal[timp]=3+np.random.randn(len(timp))*0.2
    >>> signal=sigp.lfilter([1],[1,0.9],signal); signal+=np.random.randn(len(signal))*0.03
    >>> #plt.figure(); plt.plot(signal)
    >>> signal=sigp.decimate(signal,10)
    >>>
    >>> # Synchronisation
    >>> synchr,delta=fb.synchronisation2(signal,100,method='maxCxyint',param=1,scaleopt='unbiased',compens=True)
    >>>
    >>> # Graphs
    >>> pos=np.cumsum(100*np.ones(len(delta)))-100;pos=pos-delta;pos=np.array(pos,int);
    >>> plt.figure(); plt.plot(signal);pos=pos[0:-1];plt.plot(pos+50,signal[pos+50],'x')
    >>> plt.figure(); fb.supPlot(synchr,100)

    """
   
    # Creation      : Monday 1st December 2003 (MATLAB version)
    # Modifications : Thursday 31 January 2019 (Translation to Python 3.6)
    #                 Monday 4 February 2019 : Add graph
    #                 Thursday 7 February 2019 : Docstring in NumPy format
    #                                      and use bool for estsync and compens
    #                                      trigger method becomes threshold
    #                                      creates rthreshold (threshold + rising edge)
    #                                      suppress dec to have + or - shifts
    #                 Wednesday 27 February 2019 : circ option for correlation
    #                                              modulo period for graph=2
    #                 Wednesday 1st April 2020 (Auto-test)
    # Version       : 1.1 i


    # Check parameters

    if method not in ('maxCxy','baryCxy','maxCxyint','threshold','rthreshold','max','ceps'):
        raise ValueError('Illegal value for method.')
        
    if method in ('rthreshold','threshold') and param==None:
        raise ValueError('You should give a threshold in param for threshold and trigger methods')
        
    # Interpret period parameter   
    if not np.isscalar(period):
        reference=period
        period=len(reference)
    else:
        reference=signal[0:period]

    # Normalisation factor for correlation
    if scaleopt=='none':
        normCorr=1
    elif scaleopt=='biased':
        normCorr=period
    elif scaleopt=='unbiased':
        tau = np.arange(-(period - 1), period)
        normCorr=period-abs(tau)
    elif scaleopt=='circ':
        normCorr=1
    else:
        raise ValueError('Illegal value for scaleopt')
        
    # Estimation of shifts between each periods in the signal
    progressBar(0,"1/2")
    t=np.arange(period)   # signal[t] is compared to reference signal
    offset=0
    delta=[]              # List to store the estimated shifts
        
    while t[-1]<len(signal):
        # Compute correlation if necessary
        if method in ('maxCxy','baryCxy','maxCxyint'):
            if scaleopt=='circ':
                correlation=sigp.correlate(reference,np.tile(signal[t],2),'valid')
            else:
                # https://stackoverflow.com/questions/43652911/python-normalizing-1d-cross-correlation
                correlation=sigp.correlate(reference,signal[t],'full')/normCorr
            
          
        # Lag estimation
        if method=='maxCxy':
            posmax=np.argmax (correlation)
            delta.append(posmax-period+offset)
           
        elif method=='baryCxy':
            weight=np.abs(correlation)
            xi=np.arange(1,2*period)
            barycenter=np.round (np.sum(weight*xi)/np.sum(weight))
            delta.append(np.round(barycenter)-period+offset)
           
        elif method=='maxCxyint':
            interpol=interp.interp1d(np.arange(len(correlation)),correlation,'cubic')
            icorrelation=interpol(np.arange((len(correlation)-1)*10)/10)
            posmax=np.argmax(icorrelation)/10
            delta.append(posmax-period+offset)

        elif method=='threshold':
            #dec=min(period//2,t[0]) # To have + or - shifts use t-dec instead of t
            dec=0
            indice=np.where(signal[t-dec]>=param)[0]-dec
            if len(indice)!=0:
                delta.append(-indice[0]+offset)
            else:
                delta.append(0)
           
        elif method=='rthreshold':
            #dec=min(period//2,t[0]) # To have + or - shifts use t-dec instead of t
            dec=0
            indice=np.where(np.logical_and(signal[t-dec+2]-signal[t-dec]>param[1]/2,signal[t-dec+1]>=param[0]))[0]-dec
            if len(indice)!=0:
                delta.append(-indice[0]+offset)
            else:
                delta.append(0)
           
        elif method=='max':
            #dec=min(period//2,t[0]) # To have + or - shifts use t-dec instead of t
            dec=0 # no negative shifts
            indice=np.argmax (signal[t-dec])
            indice=indice-dec
            delta.append(-indice+offset)
           
        elif method=='ceps':
            prelev=np.concatenate((reference,signal[t],np.zeros(2*period)))
            # cepstre=np.real(np.fft.ifft(np.log(np.abs(np.fft(prelev)))))
            cepstre=np.real(np.fft.ifft(np.log(np.abs(np.fft.fft(sigp.correlate(prelev,prelev,'full')**2)))))
            cepstre=cepstre[0:(len(cepstre)//2)]
            cepstre[0:period//2+1]=0
            position,amp=peakDetection(cepstre,'diffInterp',10,10,10)
            position=np.array(position)
            sel=np.argmin(abs(position-period))
            position=position[sel]
            delta.append(position-1-period+offset)
       
        # Slip compensation
        if compens:
            compensation=delta[-1]-offset
            if abs(compensation)>1:
                compensation=int (compensation)
                t=t-compensation
                offset=offset+compensation
        else:
            compensation=0
    
        t=t+period
       
        progressBar(t[0]/len(signal),"1/2")
       
    progressBar(-1,'')
    delta=np.array(delta)

    # Signal shifting
    #delta=delta-delta[0]  
    if estsync:
        delta2=delta-delta[0]
        progressBar(0,"2/2")
        synchr=np.zeros((len(delta2),period))
        t=np.arange(period)
        for index in range(len(delta2)):
            if (t[-1]-int(delta2[index]))<len(signal):
                extrait=signal[t-int(delta2[index])]
                # decdec is the decimal part of shifting
                decdec=delta2[index]-int(delta2[index])
                if decdec!=0:
                    extrait=circShift (extrait,decdec)
                synchr[index,:]=extrait[0:period]
            t=t+period
            progressBar(index/len(delta2),"2/2")
        progressBar(-1,'')
    else:
        synchr=None
    
    if graph & 1==1:
        plt.figure()
        supPlot(synchr,scale='ind')
        plt.suptitle('Synchronisation : stack synchronised blocks')

    if graph & 2==2:
        plt.figure()
        plt.subplot(1,2,1)
        periodPlot(signal,period)
        plt.plot((-delta) % period,np.arange(len(delta))+1.5,'r.--')
        plt.title('Before')
        plt.subplot(1,2,2)
        supPlot(synchr,scale='ind')
        plt.title('After')
        plt.suptitle('Synchronisation : stack synchronised blocks')
        
    if graph & 4==4:
        plt.figure()
        colors=['r','b','k','c']
        N=len(signal)
        plt.plot(signal)
        ax=plt.gca()
        delta2=delta-delta[0]
        for index in range(len(delta2)):
            x1=index*period-delta2[index]
            x2=x1+period
            if x1>=0 and x1<N-period:
                y1=np.min(signal[int(x1):int(x2)])
                y2=np.max(signal[int(x1):int(x2)])
            else:
                y1=np.min(signal)
                y2=np.max(signal)
            ax.add_patch(plt.Rectangle((x1,y1),x2-x1,y2-y1,linewidth=1,edgecolor=colors[index % 4],facecolor='none'))
        plt.suptitle('Synchronisation : block position')

    return synchr,delta
    
def progressBar(position,step):
    """
    Display a text progress bar
    
    Parameters
    ----------
    
    position   : between 0 and 1
                 otherwise suppress the bar
    """
    
    if position>=0 and position<=1:
        nb_done=int(np.ceil(position*50))
        print("\rsynchro"+step+" : \u001b[44;1m"+">"*nb_done+"\u001b[0m\u001b[44m"+"-"*(50-nb_done)+"\u001b[0m",end='')
    else:
        print("\r"+" "*70+"\r\u001b[A")

# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("You should see a two figure.")
    print("Figure 1 : a noisy signal with peak. Peak are marked with a cross.")
    print("Figure 2 : a superposition of these peak. Peak are verticaly aligned.")

    # Generate test signal
    timp=np.array(np.cumsum(np.random.rand(30)*10+1000),int)-500
    signal=np.zeros(int(max(timp))+500)
    signal[timp]=3+np.random.randn(len(timp))*0.2
    signal=sigp.lfilter([1],[1,0.9],signal); signal+=np.random.randn(len(signal))*0.03
    #plt.figure(); plt.plot(signal)
    signal=sigp.decimate(signal,10)

    # Synchronisation
    synchr,delta=synchronisation2(signal,100,method='maxCxyint',param=1,scaleopt='unbiased',compens=True)

    # Graphs
    pos=np.cumsum(100*np.ones(len(delta)))-100;pos=pos-delta;pos=np.array(pos,int);
    plt.figure(); plt.plot(signal);pos=pos[0:-1];plt.plot(pos+50,signal[pos+50],'x')
    plt.figure(); supPlot(synchr,100)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
=======
Display
=======

Contents
--------
Functions to display data (superposition, markers, ...):

Details
-------

periodPlot
    Superpose periods of an signal in one plot
plotMatrix
    Draw an picture associated to a matrix and provide a tool to adjust visualisation
supPlot
    Display a set of signal by overlapping them
verticalLabel
    Put vertical labeled lines on the figure
 
Note
----
Version 2020.03 31-Mar-2020
Copyright (c) 2001-2020 Frédéric BONNARDOT.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.


Warning
-------

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim damages or other liability.

"""

__author__ = "Frédéric BONNARDOT"
__copyright__ = "Copyright 2020, Frédéric BONNARDOT"
__credits__ = "Frédéric BONNARDOT"
__license__ = "AGPL-3.0-or-later license"
__version__ = "2020.03"
__maintainer__ = __author__
__email__ = "frederic.bonnardot@univ-st-etienne.fr"
__status__ = "Prototype"

__all__=[
        'periodPlot',
        'plotMatrix',
        'supPlot',
        'verticalLabel'
]

from .periodPlot import periodPlot
from .plotMatrix import plotMatrix
from .supPlot import supPlot
from .verticalLabel import verticalLabel


# -*- coding: utf-8 -*-
"""
Created on Sat Feb  2 13:44:24 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2002-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                       initial after the name) 
"""

import numpy             as np                 # matrix management

def circShift (datas,shift,forceft=False):
    """
    Circular shift of datas (use a Fourier Transform if shift is not an integer).

    Parameters
    ----------
    datas : vector or matrix(1 signal = 1 row for matrix) use np.array
        data to shift 
        
    shift : vector or float
        decal=datas [t-shift]
        
    forceft : bool, optional
        if True, force to use the Fourier Transform
        optional, False by default

    Return
    ------
    decal : vector or matrix (1 signal = 1 row for matrix)
        shifted datas
    
    Note
    ----
    The computing are faster if the number of samples are in the form of :math:`2^n` with n integer.

    Examples
    --------
    Example 1 :
        
    >>> import fbonnardot as fb; import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> a=np.concatenate((np.arange(65),np.arange(63,0,-1)))
    >>> line,=plt.plot (fb.circShift(a,0/10-6))
    >>> for d in range(1,129):
    >>>     line.set_ydata(fb.circShift(a,d/10-6))
    >>>     plt.pause(0.01)

    Example 2 :
        
    >>> import fbonnardot as fb; import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> a=np.concatenate((np.arange(65),np.arange(63,0,-1)))
    >>> a=np.array([a,a])
    >>> for b in range(1,101):
    >>>     fb.supPlot (fb.circShift(a,[b/10,-b/10]))
    >>>     plt.pause(0.01)
    >>>     plt.gca().clear()
    """
    
    # Creation              : Monday 8 April 2002      (MATLAB version)
    # Modifications         : Saturday 2 February 2019 (Translation to Python 3.6)
    #                         Thursday 6 February 2919 (Docstring in NumPy format and change forceft to bool)
    #                         Wednesday 1st April 2020 (Docstrings and auto-test)
    # Version               : 1.1 i

    # Check parameters
    if type(datas) is not np.ndarray:
        raise ValueError("datas should be numpy arrays")
    
    if datas.ndim==1:
        decal=circShift(np.array([datas]),shift,forceft)
        return decal[0]
    else:
         li,col=datas.shape
    
    if np.isscalar(shift):
        shift=np.ones(li)*shift
  
    if len(shift)!=li:
        raise ValueError("If shift is a vector it must have its size equal to signals numbers.")
     
    decal=np.zeros(datas.shape)
        
    # Pre computing for Fourier Transform
    halfcol=col//2
    if col & 1==0:
        # Even samples number
        # ex. 10 => f=[0 1 2 3 4 5 -4 -3 -2 -1]
        f=np.concatenate((np.arange(0,halfcol+1),np.arange(-halfcol+1,0)))/col
    else:
        # Odd samples number
        # ex.  9 => f=[0 1 2 3 4 -4 -3 -2 -1]
        f=np.concatenate((np.arange(0,halfcol+1),np.arange(-halfcol,0)))/col
    
    # Pour each signal (i.e. line)
    for index in range(li):
        sht=shift[index]
        # Two case :
        if sht==np.floor (sht) and forceft==False:
            # Integer shift
            # Replace a negative shift by a positive shift
            while sht<0:
                sht=sht+col
            # Set sht<col
            sht=int(sht % col)
            # Shift
            decal[index,sht:col]=datas[index,0:(col-sht)]
            decal[index,0:sht]  =datas[index,(col-sht):col]
        else:
            # Real shift => use Fourier Transform
            # Use FT [s (t-tau)]=TFT[s (t)].e^[-2.i.pi.f.tau]
            #   with phasis=e^[-2.i.pi.f.tau]
            # Compute FT
            tf=np.fft.fft (datas[index,:])
            # Compute phasis
            phase=np.exp(-2j*np.pi*f*sht)
            # Apply phase to FT
            tf=tf*phase
            # Compute inverse FT
            # If datas is real,
            if np.isreal(datas[index,0]):
                # decal should be real
                decal[index,:]=np.real(np.fft.ifft (tf))
            else:
                decal[index,:]=np.fft.ifft (tf)
    
    return decal

# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    import matplotlib.pyplot as plt
    print("Auto-test if Python script launched from console")
    print("You should see an animated figure with a moving triangle wave.")
    a=np.concatenate((np.arange(65),np.arange(63,0,-1)))
    line,=plt.plot (circShift(a,0/10-6))
    for d in range(1,129):
        line.set_ydata(circShift(a,d/10-6))
        plt.pause(0.01)
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jan 15 17:54:47 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2016-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                       initial after the name)                                                   initial after the name)
"""

import numpy as np                       # matrix management
import matplotlib.pyplot as plt          # plot
import scipy.signal as sig               # Signal processing fonctions
import warnings

def globalMinMax (datas,method='savgol',pmeth=None,badrem='period',enh=None,penh=None,graph=0):
    """
    Detection of locals minimum and maximum.

    Parameters
    ----------
    datas : matrix or vector,
        - nb_sig signals of N samples stored in column (Nxnb_sig matrix)
        - or vector if only one signal
                    
    method : string, optionnal
        method to extimate min and max location
            - 'diff' : use diff to estimate the 1st and 2nd derivative
            - 'poly' : use a local polynomial regression to estimate the derivative
                be careful, 1 polynomial is estimated per sample
            - 'savgol' : use Savitzky-Golay to estimate the local polynomial
                approx. 100 times faster than poly
        optional, savgol by default
                  
    pmeth : string, optionnal
        parameters for method
            - 'diff' : None (no parameter required)
            - 'poly' : 2 elements [delta,order] where
                - order is the polynomial order
                - 2*delta+1 is the number of samples used for regression
                         if value is None, use [10;2]
            - 'savgol' : same as poly
        optional, None by default
                  
    badrem : string, optionnal
        enable to suppress false detection
            - None : no false detection suppression
            - 'period' : compute distance between consecutive mininums
                partition this distance in two groups (false and exact)
                remove points associated in false group
                next do the same for maximums
        optional, 'period' by default
                  
    enh : string, optionnal
        enhance position detection of minimum and maximum
        tupple of two value to set up pmin and pmax
            or one value to apply to pmin and pmax
        - None     : no enhancement
        - 'zak'    : method proposed by F. Zakaria in its thesis
                       start with  given minimum
                       compare with delta next samples
                         minimum if the serie is ascending without descending
                         else update minimum position to the left reiterate
                        ascending or descending is evaluated with the derivative
                        estimated previously
        optional, None by default
                  
    penh : string, optionnal
        parameter or tuple of parameter for enh
            - 'zak' : give value of delta, if None delta is set to 30
        optional, None by default

    graph : scalar,
        - 1 to display a figure to show the result (only if nb_sig=1)
        - 0 else
        optional, 0 by default
            
    Return
    ------
    pmin : array
        - array of containing nb_sig arrays of minimum positions
        - or just an array of minimum position if datas is a vector
    
    pmax : array
        - array of containing nb_sig arrays of maximum positions
        - or just an array of minimum position if datas is a vector

    Example
    -------
    >>> import fbonnardot as fb; import numpy as np; import scipy as sp
    >>> pvar=100; pmin=20; data=np.array([])
    >>> for it in range(10):
    >>>     data=np.concatenate([data,np.ones(int(np.random.rand()*pvar)+pmin),
                    np.ones(int(np.random.rand()*pvar)+pmin)*-1])
    >>> dataf=sp.signal.lfilter([1,0],[1,-np.exp(-2*np.pi/30)],data)+np.random.randn(len(data))*0.1
    >>> pmin,pmax = fb.globalMinMax (dataf,enh='zak',graph=1)

    Note
    ----
    For method 'diff' and 'poly' :
        - a maximum local is detected when derivative goes from - to + and
             amplitude of data is greater than 0.3 (on normalized data)
        - a maximum local is detected when derivative goes from - to + and
             amplitude of data is lower than 0.7 (on normalized data)
             
    Bibliography
    ------------
    - https://fr.wikipedia.org/wiki/Algorithme_de_Savitzky-Golay
    - Abraham Savitzky et Marcel J. E. Golay, « Smoothing and Differentiation of Data by Simplified Least Squares Procedures », Analytical Chemistry, vol. 8, no 36,‎ 1964, p. 1627–1639 (DOI 10.1021/ac60214a047)
    - Human locomotion analysis : exploitation of cyclostationarity properties of signals, F. Zakaria (http://www.theses.fr/2015STET4019)
    """
    
    # Creation              : Friday 2 September 2016 (MATLAB version)
    # Modifications         : Monday 15 January 2018  (Translation to Python 3.6)
    #                         Thursday 31 January 2018 (in enhance detection
    #                            remove close samples instead of remove identical samples)
    #                         Wednesday 1st April 2020 (Docstrings and auto-test)
    # Version               : 1.0 i

    def part_2 (signal,nb_iter,min_s):
        """ Partition in 2 clusters signal with min and max as initial position
               next, update position of the 2 clusters (k-means)
            If a cluster have less than t_min elements, suppress elements from
               the set and try again
        """
        ec1=np.min(signal); ec2=np.max(signal);

        iter=0
        while iter<nb_iter:
            center=(ec1+ec2)/2
            pc1=np.where(signal<center)[0]; pc2=np.where(signal>center)[0]
            # Is the number of elements of cluster significant ?
            if len(pc1)>min_s and len(pc2)>min_s:
                # Update the center of clusters
                ec1=np.mean(signal[pc1]); ec2=np.mean(signal[pc2])
            else:
                # A cluster have non significative element-> suppress them
                if len(pc1)<=min_s:
                    signal=signal[pc2]
                else:
                    signal=signal[pc1]
                ec1=np.min(signal); ec2=np.max(signal);
                iter=iter-1 # Cancel iteration
            iter=iter+1
        
        return ec1,ec2
    
    ########

    if np.ndim(datas)>1:
        # Case where datas is a matrix => call nsig time this algo with vector datas
        N,nb_sig = datas.shape
        if nb_sig>N:
            warnings.warn ('Be careful : it seems that you have transposed datas (number of signals>signal size) !')
        pmin=[]
        pmax=[]
        for index in range(nb_sig):
            lpmin,lpmax=globalMinMax(datas[:,index],method,pmeth,badrem,enh,penh,0)
            pmin.append(lpmin)
            pmax.append(lpmax)

        return pmin, pmax
    
    N=len(datas)

    if pmeth==None and (method=='poly' or method=='savgol'):
        pmeth=[10,2]

    if type(enh) is not list and type(enh) is not tuple:
        enh=(enh,enh)

    if type(penh) is not list and type(penh) is not tuple:
        penh=(penh,penh)
    
    
    # Normalization of signal between 0 and 1 :
    #   we work in area 0.1 N to 0.9 N to be protected of border effect
    nn=np.arange(int(0.1*N),int(0.9*N))
    vmax=np.max(datas[nn])
    vmin=np.min(datas[nn])
    
    data_norm=(datas-vmin)/(vmax-vmin)
    
    # Search of minimum and maximum
    
    # Derivative bases method
    if method=='poly' or method=='diff' or method=='savgol':
        if method=='diff':
            deriv =np.concatenate([np.diff(data_norm), [0]])
            #deriv2=np.concatenate([np.diff(deriv)    , [0]])
        elif method=='poly':
            # Polynomial fit and deduce derivative
            delta=pmeth[0]
            order=pmeth[1]
            deriv =np.zeros(N)
            #deriv2=np.zeros(N)
            # At the begining we do not have delta previous samples -> we do our best
            for index in range(delta):
                # Use x=0 where we want to compute the derivative
                coeffs=np.polyfit(np.arange(-index,delta+1),data_norm[0:index+delta+1],order)
                deriv [index]=coeffs[-2]
                #deriv2[index]=coeffs[-3]/2
            # After we can compute using sampled before and after
            for index in range(delta,N-delta):
                coeffs=np.polyfit(np.arange(-delta,delta+1),data_norm[np.arange(index-delta,index+delta+1)],order)
                deriv [index]=coeffs[-2]
                #deriv2[index]=coeffs[-3]/2
            # At the end, some coefficient after are also missing
            for index in range(N-delta,N):
                coeffs=np.polyfit(np.arange(-delta,N-index),data_norm[np.arange(index-delta,N)],order)
                deriv [index]=coeffs[-2]
                #deriv2[index]=coeffs[-3]/2
        elif method=='savgol':
            # Polynomial fit and deduce derivative
            delta=pmeth[0]
            order=pmeth[1]
            deriv =sig.savgol_filter(data_norm,2*delta+1,order,deriv=1)
            #deriv2=sig.savgol_filter(data_norm,2*delta+1,order,deriv=2)
    
        # For each methods based on derivative
        # Find extremum by zero crossing detection
        t=np.arange(0,N-1)
        pmax=2+np.where(np.logical_and(np.logical_and(deriv[t]>=0,deriv[t+1]<0),data_norm[t]>0.3))[0]
        pmin=2+np.where(np.logical_and(np.logical_and(deriv[t]<0,deriv[t+1]>=0),data_norm[t]<0.7))[0]
    else:
        raise ValueError('Unknown method')
    
    # Suppress false detection
    if badrem=='period':
        # Compute the "period" between 2 minimum and partition in 2 groups
        ecart=np.diff(pmin)
        ec1,ec2=part_2(ecart,nb_iter=3,min_s=int(np.ceil(len(ecart)/10)))
        # Filter if ec1<0.5 ec2 (i.e. there is 2 distinct periods)
        if ec1<0.5*ec2:
            center=(ec1+ec2)/2
            pok=1+np.where(ecart>=center)[0]
            pmin=pmin[pok]
        # Same for pmax
        ecart=np.diff(pmax)
        ec1,ec2=part_2(ecart,nb_iter=3,min_s=int(np.ceil(len(ecart)/10)))
        if ec1<0.5*ec2:
            center=(ec1+ec2)/2
            pok=1+np.where(ecart>=center)[0]
            pmax=pmax[pok]
    elif badrem!=None:
        raise ValueError ('Unknown method for badrem')
            
    # Enhance detection
    if enh[0]=='zak':
        deltaz=penh[0]
        if deltaz==None:
            deltaz=30
        for it in range(len(pmin)):
            pos=pmin[it]
            t=np.arange(pos,min(N-1,pos+deltaz))
            while np.count_nonzero(deriv[t]<0)!=0 and pos+deltaz<N-1:
                pos=pos+1
                t=np.arange(pos,min(N-1,pos+deltaz))
            pmin[it]=pos
        # Remove close values (i.e. a position that differs of more than 4 samples)
        pmin=pmin[np.where(np.diff(pmin)>4)[0]]
    elif enh[0]!=None:
        raise ValueError('Unknown method for enh')
        
    # Enhance detection
    if enh[1]=='zak':
        deltaz=penh[1]
        if deltaz==None:
            deltaz=30
        for it in range(len(pmax)):
            pos=pmax[it]
            t=np.arange(pos,min(N-1,pos+deltaz))
            while np.count_nonzero(deriv[t]>0)!=0 and pos+deltaz<N-1:
                pos=pos+1
                t=np.arange(pos,min(N-1,pos+deltaz))
            pmax[it]=pos
        # Remove close values (i.e. a position that differs of more than 4 samples)
        pmax=pmax[np.where(np.diff(pmax)>4)[0]]
    elif enh[1]!=None:
        raise ValueError('Unknown method for enh')
    
    if graph==1:
        plt.plot(datas,'k')
        if method=='diff' or method=='poly' or method=='savgol':
            plt.plot(deriv,'g')
        plt.plot(pmax,datas[pmax],'or')
        plt.plot(pmin,datas[pmin],'ob')
        if method=='diff' or method=='poly' or method=='savgol':
            if method=='savgol':
                liss=sig.savgol_filter(datas,2*delta+1,order,deriv=0)
                plt.plot(liss,'r')
                plt.legend (['signal','derivative','max','min','smoothing'])
            else:
                plt.legend (['signal','derivative','max','min'])
            plt.plot ([1,N],[0,0],'g--')

    return pmin, pmax    

# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("A noisy rectangular signal is displayed.")
    print("A smoothed version is displayed.")
    print("Derivative is displayed.")
    print("Min and max are indicated.")
    
    pvar=100; pmin=20; data=np.array([])
    for it in range(10):
         data=np.concatenate([data,np.ones(int(np.random.rand()*pvar)+pmin),
                    np.ones(int(np.random.rand()*pvar)+pmin)*-1])
    dataf=sig.lfilter([1,0],[1,-np.exp(-2*np.pi/30)],data)+np.random.randn(len(data))*0.1
    pmin,pmax = globalMinMax (dataf,enh='zak',graph=1)
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 12 11:03:21 2019

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2017-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                    initial after the name)
"""

import matplotlib.pyplot as plt          # plot
import matplotlib        as mpl
import numpy             as np           # manage vectors

def verticalLabel (x,text,color='b',linestyle='-',linewidth=0.5,position='top',fontsize=10,orient='vert',ax=None):
    """
    Put vertical labeled lines on the figure.

    Parameters
    ----------
    x : scalar or vector for more marks
        abcissa
        
    text : string
        text to display (use %i for display mark number (start at 1))
        
    color : string, optional
        color
        optional 'b' by default
        
    linestyle : string, optional
        style '-' or '--' or '-.' or ':' or 'None' ...
        optional '-' by default
        
    linewidth : float, optional
        width
        optional 0.5 by default
        
    position : string, optional
        'top' or 'bottom'
        optional, 'top' by default
        
    fontsize : float, optional
        font size, optional, 10 by default
        
    orient : string, optional
        'vert' or 'horiz'
        vert by default
        
    ax : Axes, optional
        The axes to draw to (None means current axis given by plt.gca())
        optional, None by defaut

    Return
    ------
    h : list
        list of lines added

    Example
    -------
    >>> import fbonnardot as fb; import numpy as np; import matplotlib.pyplot as plt
    >>> plt.figure()
    >>> Ts=0.1; freq=0.3
    >>> tps=np.arange(0,5,Ts); signal=np.sin(2*np.pi*tps*freq)
    >>> plt.plot(tps,signal)
    >>> fb.verticalLabel(1/freq,'period','r','--',1,'top')
    >>> fb.verticalLabel(0.5/freq,'1/2 period',[0.4,0.4,0],'-',3,'bottom',13)
    >>> fb.verticalLabel(np.arange(1,5),'{:d} sec',[0.5,0.5,0.95],'--',0.5,'top',10,'horiz')
    """

    # Creation              : Monday 12 December 2017 (MATLAB version)
    # Modifications         : Tuesday 12 March 2019   (Python translation)
    #                         Tuesday 31 March 2020 (NumPy docstring - autotest)
    # Version               : 1.1 i
    
    # For drawing in Python :
    # http://www.python-simple.com/python-matplotlib/dessin-matplotlib.php
    
    if ax==None:
        ax=plt.gca()
        
    if np.isscalar(x):
        x=[x]
        
    YLim=ax.get_ylim()
    h=[]
    for index in range(len(x)):
        h.append(ax.add_artist(mpl.lines.Line2D((x[index],x[index]),YLim, color = color,linestyle=linestyle,linewidth=linewidth)))
        #% Ins�re la ligne sous les autres �l�ments
        #chH = get(gca,'Children');
        #set(gca,'Children',[chH(2:end);chH(1)]);
        if position=='top':
            if orient=='vert':
                h.append(ax.add_artist(mpl.text.Text(x[index],YLim[1],text.format(index+1),color=color,horizontalalignment='right',verticalalignment='top',rotation=90,fontsize=fontsize)))
            else:
                h.append(ax.add_artist(mpl.text.Text(x[index],YLim[1],text.format(index+1),color=color,horizontalalignment='center',verticalalignment='top',rotation=0,fontsize=fontsize)))
        else:
            if orient=='vert':
                h.append(ax.add_artist(mpl.text.Text(x[index],YLim[0],text.format(index+1),color=color,horizontalalignment='right',verticalalignment='bottom',rotation=90,fontsize=fontsize)))
            else:
                h.append(ax.add_artist(mpl.text.Text(x[index],YLim[0],text.format(index+1),color=color,horizontalalignment='center',verticalalignment='bottom',rotation=0,fontsize=fontsize)))
 
    return h
 
# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("You should see a graph with a sine wave, vertical labels each second, at period and half period.")
    plt.figure()
    Ts=0.1; freq=0.3
    tps=np.arange(0,5,Ts); signal=np.sin(2*np.pi*tps*freq)
    plt.plot(tps,signal)
    verticalLabel(1/freq,'period','r','--',1,'top')
    verticalLabel(0.5/freq,'1/2 period',[0.4,0.4,0],'-',3,'bottom',13)
    verticalLabel(np.arange(1,5),'{:d} sec',[0.5,0.5,0.95],'--',0.5,'top',10,'horiz')   
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
=========
Detection
=========

Contents
--------
Methods for detection:

Details
-------

peakDetection
     Detect peaks in a signal
globalMinMax
     Detection of locals minimum and maximum.
 
Note
----
Version 2019.04 01-Apr-2020
Copyright (c) 2001-2020 Frédéric BONNARDOT.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

 
Warning
-------

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim damages or other liability.

"""

__author__ = "Frédéric BONNARDOT"
__copyright__ = "Copyright 2020, Frédéric BONNARDOT"
__credits__ = "Frédéric BONNARDOT"
__license__ = "AGPL-3.0-or-later license"
__version__ = "2020.04"
__maintainer__ = __author__
__email__ = "frederic.bonnardot@univ-st-etienne.fr"
__status__ = "Prototype"

__all__=[
        'peakDetection',
        'globalMinMax'
]

from .globalMinMax  import globalMinMax
from .peakDetection import peakDetection



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 31 17:35:02 2020

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This file was created by using the tutorial at
http://sametmax.com/creer-un-setup-py-et-mettre-sa-bibliotheque-python-en-ligne-sur-pypi/
"""

from setuptools import setup, find_packages
 
import fbonnardot
 
setup(
 
    # Name of library for pypi
    name='fbonnardot',
 
    # Version of code
    version=fbonnardot.__version__,
 
    packages=find_packages(),
 
    author=fbonnardot.__author__,
 
    author_email=fbonnardot.__email__,
 
    # Short description
    description="Signal processing tools for Python.",
 
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
 
    # Packages needed
    install_requires= ["numpy","scipy","matplotlib","PyQt5"],
 
    # Use file MANIFEST.in
    include_package_data=True,
 
    # Link with github page
    url='https://github.com/fredujm/python_share',
 

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 1 - Planning",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Scientific/Engineering"],
 
 
 
    # A fournir uniquement si votre licence n'est pas listée dans "classifiers"
    # ce qui est notre cas
    license="GNU Affero General Public License v3 or later (AGPLv3+)",

 
)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 26 15:40:02 2018

@author: Frédéric BONNARDOT, AGPL-3.0-or-later license
(c) Frédéric BONNARDOT, 2001-2020

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

This code is given as is without warranty of any kind.
In no event shall the authors or copyright holder be liable for any claim
                                                   damages or other liability.

If you change or adapt this function, change its name (for example add your
                                                  initial after the name)
"""

import numpy as np                       # matrix management
import scipy.interpolate as interp       # interpolation
import warnings
import matplotlib.pyplot as plt          # Plot
#from .supPlot import supPlot
import fbonnardot.display.supPlot

plt.rcParams['toolbar'] = 'toolmanager'  # Pour ajouter une toolbar

def periodPlot(data,period,selection=0,moments=0,orient='vert',overlap=0,Ts=1,scale='ind',axes=None,line_opts=None):
    """
    Superpose periods of an signal in one plot
    
    Parameters
    ----------
    data : vector (np.array)
        Datas to plot
        
    period : integer or float
        period of the data to plot
        if the period is not integer, an resampling will be done before splitting the datas
        
    selection : integer or vector, optionnal
        - if integer  :
              - 0 for showing all periods
              - n for showing the period no. n^i
              - -n for automatic selection (n biggest difference with the synchronous averaging).
        - if vector  : periods to plot
        optionnal, 0 by default
        
    moments : integer, optionnal
        - 0 to display no moments
        - 1 for synchronous average (on top)
        - n for the n first moments (on top the higher curve is the n moment)
        - -n for the cumulant of n order limited to n<=4
        optionnal, 0 by default
        
    orient : string, optionnal
        Angle of the name of the signals
            - 'horiz' : for horizontal
            - 'vert'  : for vertical
            - 'no'    : no signal name and block n° displayed
        optionnal, 'vert' by default
        
    overlap : integer, optionnal
        Overlap between signals
            - 0   by default
            - -1 for superposing the signals
        optionnal, 0 by default
        
    Ts : float, optionnal
        Sampling period
        optional, 1 by default
        
    scale : string, optionnal
        Scale of signals
            - 'ind' : each signal have its own scale to fit in the n, n+1 box
            - 'glo' : same scale for each signal
        optionnal, 'ind' by default
        
    axes : object, optionnal
        Axes to draw the datas or None to use current Axes
        optionnal, None by default
        
    line_opts : string, optionnal
        display extra curves for each lines
        optional, None by default
            - 'min'     : add minimum value
            - 'max'     : add maximum value
            - 'm3sigma' : add average-3.standard deviation
            - 'p3sigma' : add average+3.standard deviation
            - 'mean'    : add average value
               
    Returns
    -------
    array1 : vector or scalar
            - 0 if moments=0
            - mean and power of each moments if moments~=0
        in the form [mean1 power1;mean2 power2;...]
    array2 : vector
        moments or cumulants or 0
    
    Note
    ----
    The first period is period n°1
    
    Examples
    --------

    >>> import fbonnardot as fb; import numpy as np
    >>> N=1000; t=np.arange(N); period=30.223
    >>> signal=np.sin(2*np.pi*t/period)+0.1*np.random.randn(N)
    >>> fb.periodPlot(signal,period,2,2)
    
"""    
    
    # Creation              : Thursay 12 July 2001       (MATLAB version)
    # Modifications         : Wednesday 26 December 2018 (Translation to Python 3.6)
    #                         Monday 9 December 2019 (Add axes parameter)
    #                         Monday 30 December 2019 (Add line_opts parameter) 
    #                         Thursday 6 February 2020 (orient='no' option)
    #                         Tuesday 31 March 2020 (NumPy docstring - autotest)
    # Version               : 1.82 i

    # Check parameters
    if np.isscalar(selection):
        if selection==1:
            raise ValueError('Selection can\'t be 1 : 1^i=1 for all i !')

    if abs(moments)>4:
        raise ValueError('Limited to 4th order statistics.')

    if period<0:
        raise ValueError('A period can''t be negative.')
        
    if len(data)<2*period:
        raise ValueError('You need at least 2 periods.')

    if (abs(overlap)>=1 or abs(overlap)<0) and overlap!=-1:
        raise ValueError('Overlap must be in [0;1[.')
        
    names=[]
    m=None
    puiss=None

    # Case of non integer periods
    if type(period) is not int:
        warnings.warn('Not an integer period => resampling.')
        N=len(data)
        step=period/np.ceil(period)
        int_func=interp.interp1d(np.arange(1,N+1,1),data,'cubic')
        data_int=int_func(np.arange(1,N,step))
        return periodPlot (data_int,int(np.ceil(period)),selection,moments,orient,overlap,Ts/step,scale,axes,line_opts)
    else:
        # Split datas
        occurences=int(np.floor (len (data) / period))
        if occurences!=len (data)/period:
            warnings.warn ('Not a integer number of period, the last period was suppressed.')

        t=range(int(occurences*period))
        blocks=np.reshape (data[t],(occurences,period))

        # Choix of displayed blocks
        if np.isscalar(selection):
            # Scalar case for selection
            if selection==0:
                # Display all blocks
                disp_index=np.arange(occurences)
                for i in range(occurences):
                    names.append(str(i+1))
            elif selection>0:
                # selection scalar>0 => display period n° selection^i   
                nb_elem=int(np.floor(np.log(occurences)/np.log(selection))+1)
                #disp_blks=np.zeros((period,nb_elem))
                disp_index=np.zeros(nb_elem,dtype=int)
                puissanceI=1
                for i in range(nb_elem):
                    disp_index[i]=int(np.floor(puissanceI-1))
                    names.append(str(int(np.floor(puissanceI))))
                    puissanceI=puissanceI*selection
            else:
                # selection scalar<0 => automatic choice of period to display
                # computes synchronous average
                msync=np.mean(blocks,1)
                # computes residual energy for each cycle
                energy=np.zeros(occurences)
                for cycle in range(occurences):
                    # compute the residual
                    vecart=blocks[:,cycle]-msync
                    # and its energy
                    energy[cycle]=np.sum(vecart*vecart)
                # sort the energy by decreasing order - retreive the indices
                order=np.argsort(-energy)
                # keep the -selection greater
                nb=min (-selection,len(energy))
                i=np.arange(nb)
                disp_index=order[i]
                for i in range(nb):
                    names.append(str (order[i]+1))
        else:
            selection=np.array(selection)-1
            # Selection by using a vector
            if np.max(selection)>=occurences or np.min(selection)<0:
                # We do not have all asked periods - find which one
                notok=np.where(np.logical_or(selection>occurences,selection<0))[0]
                for per in notok:
                    warnings.warn('The period '+str(per)+' is not in the signal.')                  
            ok=np.where(np.logical_and(selection<occurences,selection>=0))[0]
            # Select the avalaible periods
            disp_index=selection[ok]

            for i in range(len (ok)):
                names.append(str (selection[ok[i]]+1))
    
    if line_opts!=None:
        tlo=1 # Number of elements per lines
        fmt=['k']
        if 'min' in line_opts:
            tlo=tlo+1
            fmt.append('b')
        if 'm3sigma' in line_opts:
            tlo=tlo+1
            fmt.append('b')
        if 'max' in line_opts:
            tlo=tlo+1
            fmt.append('r')
        if 'p3sigma' in line_opts:
            tlo=tlo+1
            fmt.append('r')
        if 'mean' in line_opts:
            tlo=tlo+1
            fmt.append('k--')
            
        #  sig nb in stack x stack x period
        disp_blks=np.zeros((tlo,len(disp_index),period))
        for st in range(len(disp_index)):
            if st==len(disp_index)-1:
                local_blks=blocks[disp_index[-1]:,:]
            else:
                local_blks=blocks[disp_index[st]:disp_index[st+1],:]

            # Display asked period
            disp_blks[0,st,:]=local_blks[0]
            tlo=1
            
            if 'min' in line_opts:
                disp_blks[tlo,st,:]=np.min(local_blks,axis=0)
                tlo=tlo+1
            if 'm3sigma' in line_opts:
                disp_blks[tlo,st,:]=np.mean(local_blks,axis=0)-3*np.std(local_blks,axis=0)
                tlo=tlo+1
            if 'max' in line_opts:
                disp_blks[tlo,st,:]=np.max(local_blks,axis=0)
                tlo=tlo+1
            if 'p3sigma' in line_opts:
                disp_blks[tlo,st,:]=np.mean(local_blks,axis=0)+3*np.std(local_blks,axis=0)
                tlo=tlo+1
            if 'mean' in line_opts:
                disp_blks[tlo,st,:]=np.mean(local_blks,axis=0)
                tlo=tlo+1
    else:
        disp_blks=blocks[disp_index,:]
        fmt='k'

    # Statistics (mean, cumulants, ...)
    if moments!=0:
        # Computes moments
        m=np.zeros((abs(moments),period))
        for mom in range(abs(moments)):
            m[mom,:]=np.mean(blocks**(mom+1),0)

        if moments<-1:
            # Computes cumulants
            if moments==-4:
                m[3,:]=cumulant4 (m [0,:],m [1,:],m [2,:],m [3,:])
            if moments<=-3:
                m[2,:]=cumulant3 (m [0,:],m [1,:],m [2,:])
            if moments<=-2:
                m[1,:]=cumulant2 (m [0,:],m [1,:])

        # legende
        if abs(moments)>0:
            names.append('s.av')

        if moments>1:
            for i in np.arange(2,moments+1):
                names.append('mom. '+str(i))
        elif moments<1:
            names.append('svar')
            for i in np.arange(3,-moments+1):
                names.append('cum. '+str(i))

        [li,col]=disp_blks.shape[-2:]
        i=np.arange(li+1,li+1+abs (moments))
        if line_opts==None:
            disp_blks=np.append(disp_blks,m,axis=0)
        else:
            disp_blks=np.append(disp_blks,np.tile(m,(tlo,1,1)),axis=1)
        puiss=np.zeros((abs(moments),2))
        for moy in range(abs (moments)):
            puiss[moy,0]=np.mean(m[moy,:])      
            puiss[moy,1]=np.var (m[moy,:])

    # Call supplot for display
    [li,col]=disp_blks.shape[-2:]

    fbonnardot.display.supPlot (disp_blks,col,0,Ts,names,orient,overlap,scale,axes,fmt=fmt)
    if orient!="no":
        if axes==None:
            plt.text (0,1,str(li)+' blocks   ', ha='right',va='bottom')
        else:
            axes.text (0,1,str(li)+' blocks   ', ha='right',va='bottom')

    return puiss,m

# Compute cumulants
# See the book  "Le signal aléatoire" p.52 of D. Declercq, A. Quinquis - Hermes

def cumulant4 (m1,m2,m3,m4):
    return m4-4*m1*m3-3*m2*m2+12*m1*m1*m2-6*m1*m1*m1*m1

def cumulant3 (m1,m2,m3):
    return m3-3*m1*m2+2*m1*m1

def cumulant2 (m1,m2):
    return m2-m1*m1

# Auto-test if Python script launched from console ---------------------------
if __name__ == '__main__':
    print("Auto-test if Python script launched from console")
    print("You should see a graph with a stack of noise + sine wave, synchronous average and variance.")
    N=1000; t=np.arange(N); period=30.223
    signal=np.sin(2*np.pi*t/period)+0.1*np.random.randn(N)
    periodPlot(signal,period,2,2)
    